/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistry.servlet;

import eu.europa.ec.re3gistry.constants.Constants;
import eu.europa.ec.re3gistry.module.Module;
import eu.europa.ec.re3gistry.module.ModuleMgr;
import eu.europa.ec.re3gistry.utils.Application;
import eu.europa.ec.re3gistry.utils.locale.LocalizationMgr;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;

/**
 *
 * @author francdb
 */
public class Frontend extends HttpServlet {

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType(Constants.CONTENT_TYPE);
        request.setAttribute(Constants.ERROR_KEY, null);

        //Inizializing the logger
        Logger log = Logger.getLogger(Constants.BASEAPP_NAME);
        request.setAttribute(Constants.LOG_KEY, log);

        try {

            /**
             * Initializing elements *
             */
            //Initializes the properties related to the main module
            Properties props = Application.getProperties();
            request.setAttribute(Constants.PROPS_KEY, props);

            //Initializing baseurl
            String baseurl = Application.getBasepath(request);
            request.setAttribute(Constants.BASEURL_KEY, baseurl);

            //Initializint modules
            ArrayList<Module> modules = ModuleMgr.getModules(getServletContext(), request);
            request.setAttribute(Constants.MODULES_KEY, modules);

            //Initializing resource bundle (localization)
            ResourceBundle localization = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.BASEAPP_NAME + Constants.LOCALIZATION_SUFFIX, LocalizationMgr.getCurrentLocale(request));
            request.setAttribute(Constants.LOCALIZATION_KEY, localization);

            //Setting the current locale
            Locale currentLocale = LocalizationMgr.getCurrentLocale(request);
            request.setAttribute(Constants.CURRENTLOCALE_KEY, currentLocale);

            //Setting the available language site
            List availableLanguages = LocalizationMgr.getAvailableLanguages();
            request.setAttribute(Constants.AVAILABLELANGUAGES_KEY, availableLanguages);

            /**
             * Loading request parameters *
             */
            //Module parameter
            String moduleParam = request.getParameter(Constants.MODULE_PARAMETER_KEY);
            if (moduleParam != null) {
                try {
                    //Loading the module
                    Module module = new Module(moduleParam, currentLocale);
                    request.setAttribute(Constants.MODULE_KEY, module);

                } catch (Exception e) {
                    log.error("The requested module was not found or there was an error loading it.");
                    request.setAttribute(Constants.ERROR_KEY, Constants.ERROR_NO_MODULE);
                }
            }

            //Servlet forward
            request.getRequestDispatcher(Constants.PAGE_FORWARD).forward(request, response);

        } catch (Exception e) {
            log.error("Error during the page initialization. Message: " + e.getMessage());
            request.getRequestDispatcher(Constants.ERROR_GENERIC).forward(request, response);
        } finally {
        }
    }

    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }
}
