/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Customattributevalue;
import eu.europa.ec.re3gistrycommon.model.Customattribute;
import eu.europa.ec.re3gistrycommon.model.Item;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.Query;

public class CustomattributevalueMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public CustomattributevalueMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the customattributevalue with the specified uuid.
     *
     * @param uuid the unique id of the customattributevalue
     * @return the customattributevalue with the specified uuid
     * @throws Exception
     */
    public Customattributevalue getCustomattributevalueByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Customattributevalue.findByUuid");
        q.setParameter("uuid", uuid);
        return (Customattributevalue) q.getSingleResult();
    }

    public List<Customattributevalue> findCustomAttributeValueFromItemAndCustomAttribute(Item item, Customattribute customattribute) throws Exception {
        //Checking parameters
        if (item == null || customattribute == null) {
            throw new Exception("The item or customattribute passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Customattributevalue.findCustomAttributeValueFromItemAndCustomAttribute");
        q.setParameter("item", item);
        q.setParameter("customattribute", customattribute);
        return (List<Customattributevalue>) q.getResultList();
    }

    public List<Customattributevalue> findCustomAttributeValueByItem(Item item) throws Exception {
        //Checking parameters
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Customattributevalue.findCustomAttributeValueByItem");
        q.setParameter("item", item);

        return (List<Customattributevalue>) q.getResultList();
    }
    
    public boolean checkCustomattributeByNameValue(String name, String value) throws Exception {
        //Checking parameters
        if (name == null || value == null) {
            throw new Exception("The one or both the parameter is null.");
        }
        
        //Preparing query
        Query q = this.em.createNativeQuery(Constants.SQL_CUSTOM_ATTRIBUTE_BY_NAME_VALUE);
        q.setParameter(1, name);
        q.setParameter(2, value);
        
        List results;
        try{
            results = q.getResultList();
        }catch(NoResultException e){
            return false;
        }
        
        if(results.size()>0){
            return true;
        }else{
            return false;
        }
    }

    /**
     * Insert the specified customattributevalues in the database. Returns true
     * if the operation succeed.
     *
     * @param ca the customattributevalue object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertCustomattributevalue(Customattributevalue ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattributevalue passed by parameter is null.");
        }

        if (ca.getDatecreation() == null) {
            ca.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ca);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ca);

        return true;
    }

    /**
     * Update the customattributevalue specified by parameter. Returns true if
     * the operation succeed.
     *
     * @param ca the updated customattributevalue (this method update the
     * customattributevalue on the db with the customattributevalue passed by
     * parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateCustomattributevalue(Customattributevalue ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattributevalue passed by parameter is null.");
        }

        if (ca.getDatelastupdate() == null) {
            ca.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ca, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ca);

        return true;
    }

    /**
     * Delete from the database the customattributevalue specified by parameter.
     * Returns true if the operation succeed.
     *
     * @param ca the customattributevalue do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteCustomattributevalue(Customattributevalue ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattributevalue passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ca);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Customattributevalue ca) throws Exception {
        if (ca.getUuid() == null || ca.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ca.getCustomattribute() == null) {
            throw new Exception("Customattribute field not available.");
        }

        if (ca.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ca.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Customattributevalue ca, boolean isUpdate) throws Exception {
        if (ca.getUuid() == null || ca.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ca.getCustomattribute() == null) {
            throw new Exception("Customattribute field not available.");
        }

        if (ca.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ca.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && ca.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }
}
