/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Procedurestatus;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class DataprocedureMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public DataprocedureMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns all dataimport completed between the 2 date parameters
     *
     * @param lastCompletedStaticizerDatestart
     * @param inprogressStaticizerDatestart
     * @return all the dataimport completed
     * @throws Exception
     */
    public List<Dataprocedure> findAllBetweenDatesOrderAscByDatecompleted(Date lastCompletedStaticizerDatestart, Date inprogressStaticizerDatestart) throws Exception {
        if (lastCompletedStaticizerDatestart == null && inprogressStaticizerDatestart == null) {
            throw new Exception("The lastCompletedStaticizerDatestart or inprogressStaticizerDatestart passed by parameter is null.");
        }

        Query q = this.em.createNamedQuery("Dataprocedure.findAllBetweenDatesOrderAscByDatecompleted");
        q.setParameter("dateinit", lastCompletedStaticizerDatestart);
        q.setParameter("datefinal", inprogressStaticizerDatestart);

        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * Returns all dataimport completed after the date parameters
     *
     * @param inprogressStaticizerDatestart
     * @return all the dataimport completed
     * @throws Exception
     */
    public List<Dataprocedure> findAllAfterDateOrderAscByDatecompleted(Date inprogressStaticizerDatestart) throws Exception {
        if (inprogressStaticizerDatestart == null && inprogressStaticizerDatestart == null) {
            throw new Exception("The inprogressStaticizerDatestart passed by parameter is null.");
        }

        Query q = this.em.createNamedQuery("Dataprocedure.findAllAfterDateOrderAscByDatecompleted");
        q.setParameter("date", inprogressStaticizerDatestart);

        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * @return a list of all the Dataprocedure
     *
     * @throws Exception
     */
    public List<Dataprocedure> findAll() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findAll");
        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * @return a list of Dataprocedure completed ordered ASC
     *
     * @throws Exception
     */
    public List<Dataprocedure> findAllOrderAscByDatecompleted() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findAllOrderAscByDatecompleted");
        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * @return a list of Dataprocedure by the status parameter
     * * @param procedurestatus
     *
     * @throws Exception
     */
    public List<Dataprocedure> findByProcedurestatus(Procedurestatus procedurestatus) throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findByProcedurestatus");
        q.setParameter("procedurestatus", procedurestatus);
        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * Returns the cdataimport with the specified uuid.
     *
     * @param uuid the unique id of the cdataimport
     * @return the cdataimport with the specified uuid
     * @throws Exception
     */
    public Dataprocedure getDataprocedureByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findByUuid");
        q.setParameter("uuid", uuid);
        return (Dataprocedure) q.getSingleResult();
    }

    /**
     * Insert the specified cdataimports in the database. Returns true if the
     * operation succeed.
     *
     * @param di the cdataimport object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertDataprocedure(Dataprocedure di) throws Exception {
        //Checking parameters
        if (di == null) {
            throw new Exception("@@ The Dataprocedure passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(di);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("@@ The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(di);

        return true;
    }

    /**
     * Update the cdataimport specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param di the updated cdataimport (this method update the cdataimport on
     * the db with the cdataimport passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateDataprocedure(Dataprocedure di) throws Exception {
        //Checking parameters
        if (di == null) {
            throw new Exception("@@ The Dataprocedure passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(di, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("@@ The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(di);

        return true;
    }

    /**
     * Delete from the database the cdataimport specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param di the cdataimport do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteDataprocedure(Dataprocedure di) throws Exception {
        //Checking parameters
        if (di == null) {
            throw new Exception("@@ The Dataprocedure passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("@@ The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(di);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Dataprocedure di) throws Exception {
        if (di.getUuid() == null || di.getUuid().trim().equals("")) {
            throw new Exception("@@ Uuid field not available.");
        }

        if (di.getFolder() == null) {
            throw new Exception("@@ Folder field not available.");
        }

        if (di.getDatestarted() == null) {
            throw new Exception("@@ Datestarted field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Dataprocedure di, boolean isUpdate) throws Exception {
        if (di.getUuid() == null || di.getUuid().trim().equals("")) {
            throw new Exception("@@ Uuid field not available.");
        }

        if (di.getFolder() == null) {
            throw new Exception("@@ Folder field not available.");
        }

        if (di.getDatestarted() == null) {
            throw new Exception("@@ Datestarted field not available.");
        }

        /*if (isUpdate && di.getDateend()== null) {
         throw new Exception("@@ Dateend field not available.");
         }*/
        return true;
    }
}
