/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Register;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class ItemMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public ItemMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the item with the specified uuid.
     *
     * @param uuid the unique id of the item
     * @return the item with the specified uuid
     * @throws Exception
     */
    public Item getItemByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Item.findByUuid");
        q.setParameter("uuid", uuid);
        return (Item) q.getSingleResult();
    }

    /**
     * Returns the item with the specified uriname.
     *
     * @param uriname the unique id of the item
     * @return the item with the specified uriname
     * @throws Exception
     */
    public Item getItemByUriname(String uriname) throws Exception {
        //Checking parameters
        if (uriname == null) {
            throw new Exception("The uriname passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Item.findByUriname");
        q.setParameter("uriname", uriname);
        return (Item) q.getSingleResult();
    }

    public List<Item> findItemsForRegister(Register register) throws Exception {
        List<Item> itemList = new ArrayList<Item>();
        //Checking parameters
        if (register == null) {
            throw new Exception("The register passed by parameter is null.");
        }
        List<Itemclass> itemclassList = register.getItemclassList();
        for (Iterator<Itemclass> it = itemclassList.iterator(); it.hasNext();) {
            Itemclass itemclass = it.next();

            if (itemclass.getParent() == null) {
                //Preparing query
                Query q = this.em.createNamedQuery("Item.findByItemclass");
                q.setParameter("itemclass", itemclass);

                itemList.addAll(q.getResultList());
            }
        }
        return itemList;

    }

    public int getLatestVersion(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        Query q = this.em.createNativeQuery(Constants.SQL_MAXVERSION_BY_ITEM);
        q.setParameter(1, item.getUriname());

        return (Integer) q.getSingleResult();
    }

    /**
     * Insert the specified items in the database. Returns true if the operation
     * succeed.
     *
     * @param it the item object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertItem(Item it) throws Exception {
        //Checking parameters
        if (it == null) {
            throw new Exception("The Item passed by parameter is null.");
        }

        if (it.getDatecreation() == null) {
            it.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(it);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(it);

        return true;
    }

    /**
     * Update the item specified by parameter. Returns true if the operation
     * succeed.
     *
     * @param it the updated item (this method update the item on the db with
     * the item passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateItem(Item it) throws Exception {
        //Checking parameters
        if (it == null) {
            throw new Exception("The Item passed by parameter is null.");
        }

        if (it.getDatelastupdate() == null) {
            it.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(it, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(it);

        return true;
    }

    /**
     * Delete from the database the item specified by parameter. Returns true if
     * the operation succeed.
     *
     * @param it the item do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteItem(Item it) throws Exception {
        //Checking parameters
        if (it == null) {
            throw new Exception("The Item passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(it);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Item it) throws Exception {
        if (it.getUuid() == null || it.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }
        if (it.getUriname() == null || it.getUriname().trim().equals("")) {
            throw new Exception("UriName field not available.");
        }
        if (it.getItemclass() == null) {
            throw new Exception("ItemClass field not available.");
        }
        if (it.getStatus() == null) {
            throw new Exception("Status field not available.");
        }
        if (it.getVersionnumber() < 0) {
            throw new Exception("VersionNumber field not available.");
        }
        if (it.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Item it, boolean isUpdate) throws Exception {
        if (it.getUuid() == null || it.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }
        if (it.getUriname() == null || it.getUriname().trim().equals("")) {
            throw new Exception("Uriname field not available.");
        }
        if (it.getItemclass() == null) {
            throw new Exception("Itemclass field not available.");
        }
        if (it.getStatus() == null) {
            throw new Exception("Status field not available.");
        }
        if (it.getVersionnumber() < 0) {
            throw new Exception("Versionnumber field not available.");
        }
        if (it.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && it.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }

    public Item getLatestCollection(Item item) throws Exception {
        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
        Itemcollection itemcollection = itemcollectionMgr.getLatestItemcollection(item);
        return (itemcollection != null) ? itemcollection.getCollection() : null;
    }

    public Item findByItemclassAndUriname(Itemclass itemclass, String uriname) throws Exception {
        //Checking parameters
        if (itemclass == null && uriname == null) {
            throw new Exception("The itemclass or uriname passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Item.findByItemclassAndUriname");
        q.setParameter("itemclass", itemclass);
        q.setParameter("uriname", uriname);
        return (Item) q.getSingleResult();
    }

    public List<Item> getAllItemVersionOrderDescByVersionnumber(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
        List collections = itemcollectionMgr.findByItem(item);

        Query q;
        if (collections != null && collections.size() > 0) {
            String collectionUriname = ((Itemcollection) collections.get(0)).getCollection().getUriname();
            q = this.em.createNativeQuery(Constants.SQL_ITEM_ALL_VERSION_COLLECTION, "itemResult");
            q.setParameter(1, item.getUriname());
            q.setParameter(2, item.getItemclass().getUuid());
            q.setParameter(3, collectionUriname);

        } else {
            q = this.em.createNamedQuery("Item.findByItemclassAndUrinameOrderDescByVersion");
            q.setParameter("itemclass", item.getItemclass());
            q.setParameter("uriname", item.getUriname());
        }

        List<Item> items = null;
        try {
            items = (List<Item>) q.getResultList();
        } catch (Exception e) {
            throw new Exception("Error while executing query: " + e.getMessage());
        }

        return items;

    }
}
