/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Itemclass;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class ItemclassMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public ItemclassMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the itemclass with the specified uuid.
     *
     * @param uuid the unique id of the itemclass
     * @return the itemclass with the specified uuid
     * @throws Exception
     */
    public Itemclass getItemclassByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemclass.findByUuid");
        q.setParameter("uuid", uuid);
        return (Itemclass) q.getSingleResult();
    }

    /**
     * Returns the itemclass with the specified uriname.
     *
     * @param uriname the unique uriname of the itemclass
     * @return the itemclass with the specified uriname
     * @throws Exception
     */
    public Itemclass getItemclassByUriname(String uriname) throws Exception {
        //Checking parameters
        if (uriname == null) {
            throw new Exception("The uriname passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemclass.findByUriname");
        q.setParameter("uriname", uriname);
        return (Itemclass) q.getSingleResult();
    }

    /**
     * Returns all the itemclasses ordered by dataprocedureorder.
     *
     * @return all the itemclasses ordered by dataprocedureorder.
     * @throws Exception
     */
    public List<Itemclass> getAllItemclassOrderedByDataprocedureorder() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Itemclass.findAllItemclassOrderedByDataprocedureorder");
        return q.getResultList();
    }

    /**
     * Insert the specified itemclasss in the database. Returns true if the
     * operation succeed.
     *
     * @param ic the itemclass object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertItemclass(Itemclass ic) throws Exception {
        //Checking parameters
        if (ic == null) {
            throw new Exception("The Itemclass passed by parameter is null.");
        }

        if (ic.getDatecreation() == null) {
            ic.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ic);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ic);

        return true;
    }

    /**
     * Update the itemclass specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param ic the updated itemclass (this method update the itemclass on the
     * db with the itemclass passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateItemclass(Itemclass ic) throws Exception {
        //Checking parameters
        if (ic == null) {
            throw new Exception("The Itemclass passed by parameter is null.");
        }

        if (ic.getDatelastupdate() == null) {
            ic.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ic, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ic);

        return true;
    }

    /**
     * Delete from the database the itemclass specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param ic the itemclass do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteItemclass(Itemclass ic) throws Exception {
        //Checking parameters
        if (ic == null) {
            throw new Exception("The Itemclass passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ic);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Itemclass ic) throws Exception {
        if (ic.getUuid() == null || ic.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ic.getRegister() == null) {
            throw new Exception("Register field not available.");
        }

        if (ic.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        if (ic.getOrdernumber() < 0) {
            throw new Exception("Ordernumber field not available or incorrect.");
        }

        if (ic.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Itemclass ic, boolean isUpdate) throws Exception {
        if (ic.getUuid() == null || ic.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ic.getRegister() == null) {
            throw new Exception("Register field not available.");
        }

        if (ic.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        if (ic.getOrdernumber() < 0) {
            throw new Exception("Ordernumber field not available or incorrect.");
        }

        if (ic.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && ic.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }
}
