/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Customattribute;
import eu.europa.ec.re3gistrycommon.model.Itemclasscustomattribute;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class ItemclasscustomattributeMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public ItemclasscustomattributeMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the itemclasscustomattribute with the specified uuid.
     *
     * @param uuid the unique id of the itemclasscustomattribute
     * @return the itemclasscustomattribute with the specified uuid
     * @throws Exception
     */
    public Itemclasscustomattribute getItemclasscustomattributeByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemclasscustomattribute.findByUuid");
        q.setParameter("uuid", uuid);
        return (Itemclasscustomattribute) q.getSingleResult();
    }

    /**
     * Returns the itemclasscustomattribute with the specified itemclass and
     * custom attribute.
     *
     * @param itemclass the itmeclass object
     * @param customattribute the customattribute object
     * @return the itemclasscustomattribute with the specified uuid
     * @throws Exception
     */
    public Itemclasscustomattribute getItemclasscustomattributeByItemclassCustomattribute(Itemclass itemclass, Customattribute customattribute) throws Exception {
        //Checking parameters
        if (itemclass == null) {
            throw new Exception("The itemclass passed by parameter is null.");
        }

        if (customattribute == null) {
            throw new Exception("The customattribute passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemclasscustomattribute.findByItemclassCustomattribute");
        q.setParameter("itemclass", itemclass);
        q.setParameter("customattribute", customattribute);
        return (Itemclasscustomattribute) q.getSingleResult();
    }

    public List<Itemclasscustomattribute> findAllByItemclass(Itemclass itemclass) throws Exception {
        //Checking parameters
        if (itemclass == null) {
            throw new Exception("The itemclass passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemclasscustomattribute.findAllByItemclass");
        q.setParameter("itemclass", itemclass);
        return (List<Itemclasscustomattribute>) q.getResultList();
    }

    /**
     * Insert the specified itemclasss in the database. Returns true if the
     * operation succeed.
     *
     * @param ic the itemclasscustomattribute object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertItemclasscustomattribute(Itemclasscustomattribute ic) throws Exception {
        //Checking parameters
        if (ic == null) {
            throw new Exception("The Itemclasscustomattribute passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(ic);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ic);

        return true;
    }

    /**
     * Update the itemclasscustomattribute specified by parameter. Returns true
     * if the operation succeed.
     *
     * @param ic the updated itemclasscustomattribute (this method update the
     * itemclasscustomattribute on the db with the itemclasscustomattribute
     * passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateItemclasscustomattribute(Itemclasscustomattribute ic) throws Exception {
        //Checking parameters
        if (ic == null) {
            throw new Exception("The Itemclasscustomattribute passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(ic, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ic);

        return true;
    }

    /**
     * Delete from the database the itemclasscustomattribute specified by
     * parameter. Returns true if the operation succeed.
     *
     * @param ic the itemclasscustomattribute do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteItemclasscustomattribute(Itemclasscustomattribute ic) throws Exception {
        //Checking parameters
        if (ic == null) {
            throw new Exception("The Itemclasscustomattribute passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ic);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Itemclasscustomattribute ic) throws Exception {
        if (ic.getUuid() == null || ic.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ic.getCustomattribute() == null) {
            throw new Exception("Customattribute field not available.");
        }

        if (ic.getItemclass() == null) {
            throw new Exception("Itemclass field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Itemclasscustomattribute ic, boolean isUpdate) throws Exception {
        if (ic.getUuid() == null || ic.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ic.getCustomattribute() == null) {
            throw new Exception("Customattribute field not available.");
        }

        if (ic.getItemclass() == null) {
            throw new Exception("Itemclass field not available.");
        }

        return true;
    }
}
