/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Itemsuccessor;
import eu.europa.ec.re3gistrycommon.model.Item;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class ItemsuccessorMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public ItemsuccessorMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the itemsuccessor with the specified uuid.
     *
     * @param uuid the unique id of the itemsuccessor
     * @return the itemsuccessor with the specified uuid
     * @throws Exception
     */
    public Itemsuccessor getItemsuccessorByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemsuccessor.findByUuid");
        q.setParameter("uuid", uuid);
        return (Itemsuccessor) q.getSingleResult();
    }

    public List<Itemsuccessor> findByItem(Item item) throws Exception {
        //Checking parameters
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemsuccessor.findByItem");
        q.setParameter("item", item);
        return (List<Itemsuccessor>) q.getResultList();
    }

    public List<Itemsuccessor> findBySuccessor(Item successor) throws Exception {
        //Checking parameters
        if (successor == null) {
            throw new Exception("The successor passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemsuccessor.findBySuccessor");
        q.setParameter("successor", successor);
        return (List<Itemsuccessor>) q.getResultList();
    }

    /**
     * Insert the specified itemsuccessors in the database. Returns true if the
     * operation succeed.
     *
     * @param ip the itemsuccessor object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertItemsuccessor(Itemsuccessor ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemsuccessor passed by parameter is null.");
        }

        if (ip.getDatecreation() == null) {
            ip.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ip);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ip);

        return true;
    }

    /**
     * Update the itemsuccessor specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param ip the updated itemsuccessor (this method update the itemsuccessor
     * on the db with the itemsuccessor passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateItemsuccessor(Itemsuccessor ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemsuccessor passed by parameter is null.");
        }

        if (ip.getDatelastupdate() == null) {
            ip.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ip, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ip);

        return true;
    }

    /**
     * Delete from the database the itemsuccessor specified by parameter.
     * Returns true if the operation succeed.
     *
     * @param ip the itemsuccessor do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteItemsuccessor(Itemsuccessor ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemsuccessor passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ip);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Itemsuccessor ip) throws Exception {
        if (ip.getUuid() == null || ip.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ip.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ip.getSuccessor() == null) {
            throw new Exception("Successor field not available.");
        }

        if (ip.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Itemsuccessor ip, boolean isUpdate) throws Exception {
        if (ip.getUuid() == null || ip.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ip.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ip.getSuccessor() == null) {
            throw new Exception("Successor field not available.");
        }

        if (ip.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && ip.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }

    public List<Itemsuccessor> getLatestItemsuccessor(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNativeQuery(Constants.SQL_LARESTVERSION_OF_SUCCESSOR_BY_ITEM, Itemsuccessor.class);
        q.setParameter(1, item.getUuid());

       List<Itemsuccessor> itemsuccessor = null;
        try {
            itemsuccessor = (List<Itemsuccessor>) q.getResultList();
        } catch (Exception e) {
            throw new Exception("Error while executing query: "+e.getMessage());
        }

        return itemsuccessor;
    }

    public List<Itemsuccessor> getLatestItempredecessor(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNativeQuery(Constants.SQL_LARESTVERSION_OF_PREDECESSOR_BY_ITEM, Itemsuccessor.class);
        q.setParameter(1, item.getUuid());

       List<Itemsuccessor> itempredecessor = null;
        try {
            itempredecessor = (List<Itemsuccessor>) q.getResultList();
        } catch (Exception e) {
            throw new Exception("Error while executing query: "+e.getMessage());
        }

        return itempredecessor;
    }
    
    public List<Itemsuccessor> getItempredecessorList(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemsuccessor.findBySuccessor");
        q.setParameter("successor", item);

       List<Itemsuccessor> itempredecessor = null;
        try {
            itempredecessor = (List<Itemsuccessor>) q.getResultList();
        } catch (Exception e) {
            throw new Exception("Error while executing query: "+e.getMessage());
        }

        return itempredecessor;
    }
    
     public List<Itemsuccessor> getItemsuccessorList(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemsuccessor.findByItem");
        q.setParameter("item", item);

       List<Itemsuccessor> itemsuccessor = null;
        try {
            itemsuccessor = (List<Itemsuccessor>) q.getResultList();
        } catch (Exception e) {
            throw new Exception("Error while executing query: "+e.getMessage());
        }

        return itemsuccessor;
    }
}
