/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Languagecode;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class LanguagecodeMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public LanguagecodeMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the languagecode with the specified uuid.
     *
     * @param uuid the unique id of the languagecode
     * @return the languagecode with the specified uuid
     * @throws Exception
     */
    public Languagecode getLanguagecodeByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Languagecode.findByUuid");
        q.setParameter("uuid", uuid);
        return (Languagecode) q.getSingleResult();
    }

    /**
     * return the mandatory language
     */
    public Languagecode getMasterLanguagecode() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Languagecode.findMasterLanguagecode");
        return (Languagecode) q.getSingleResult();
    }

    /**
     * Returns the languagecode for the specified isocode.
     *
     * @param isocode
     * @return the languagecode with the specified uuid
     * @throws Exception
     */
    public Languagecode findByIsocode(String isocode) throws Exception {
        //Checking parameters
        if (isocode == null) {
            throw new Exception("The isocode passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Languagecode.findByIsocode");
        q.setParameter("isocode", isocode);
        return (Languagecode) q.getSingleResult();
    }

    public List<Languagecode> findAll() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Languagecode.findAll");
        return (List<Languagecode>) q.getResultList();
    }

    /**
     * Insert the specified languagecodes in the database. Returns true if the
     * operation succeed.
     *
     * @param lc the languagecode object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertLanguagecode(Languagecode lc) throws Exception {
        //Checking parameters
        if (lc == null) {
            throw new Exception("The Languagecode passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(lc);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(lc);

        return true;
    }

    /**
     * Update the languagecode specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param lc the updated languagecode (this method update the languagecode
     * on the db with the languagecode passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateLanguagecode(Languagecode lc) throws Exception {
        //Checking parameters
        if (lc == null) {
            throw new Exception("The Languagecode passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(lc, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(lc);

        return true;
    }

    /**
     * Delete from the database the languagecode specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param lc the languagecode do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteLanguagecode(Languagecode lc) throws Exception {
        //Checking parameters
        if (lc == null) {
            throw new Exception("The Languagecode passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(lc);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Languagecode lc) throws Exception {
        if (lc.getUuid() == null || lc.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (lc.getLabel() == null) {
            throw new Exception("Label field not available.");
        }

        if (lc.getIsocode() == null) {
            throw new Exception("Isocode field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Languagecode lc, boolean isUpdate) throws Exception {
        if (lc.getUuid() == null || lc.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (lc.getLabel() == null) {
            throw new Exception("Label field not available.");
        }

        if (lc.getIsocode() == null) {
            throw new Exception("Isocode field not available.");
        }

        return true;
    }
}