/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.model.Registry;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Status;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Customattributevalue;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.apache.log4j.Logger;

public class LocalizationMgr {

    private static Logger log = Logger.getLogger(Constants.MODULE_NAME);
    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public LocalizationMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the localization with the specified uuid.
     *
     * @param uuid the unique id of the localization
     * @return the localization with the specified uuid
     * @throws Exception
     */
    public Localization getLocalizationByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Localization.findByUuid");
        q.setParameter("uuid", uuid);
        return (Localization) q.getSingleResult();
    }

    /**
     * Returns the localization list for the specific item created after date or
     * updated after date
     *
     * @param item
     * @param date
     * @return the localization with the specified uuid
     * @throws Exception
     */
    public List<Localization> findByItemAndAfterDate(Item item, Date date) throws Exception {
        //Checking parameters
        if (item == null || date == null) {
            throw new Exception("The item or the date passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Localization.findByItemAndAfterDate");
        q.setParameter("item", item);
        q.setParameter("date", date);
        return (List<Localization>) q.getResultList();
    }

    public Localization findByLanguagecodeAndStatus(Status status, Languagecode languagecode) throws Exception {
        //Checking parameters
        if (languagecode == null || status == null) {
            throw new Exception("The languagecode or status passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByLanguagecodeAndStatus");
        q.setParameter("language", languagecode);
        q.setParameter("status", status);
        return (Localization) q.getSingleResult();
    }

    public Localization findByItemclassAndLanguagecode(Itemclass itemclass, Languagecode languagecode) throws Exception {
        //Checking parameters
        if (languagecode == null || itemclass == null) {
            throw new Exception("The languagecode or itemclass passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByItemclassAndLanguagecode");
        q.setParameter("itemclass", itemclass);
        q.setParameter("language", languagecode);
        return (Localization) q.getSingleResult();
    }

    public Localization findByItemAndLanguagecode(Item item, Languagecode languagecode) throws Exception {
        //Checking parameters
        if (languagecode == null || item == null) {
            throw new Exception("The languagecode or item passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByItemAndLanguagecode");
        q.setParameter("item", item);
        q.setParameter("language", languagecode);

        return (Localization) q.getSingleResult();
    }

    public Localization findByRegisterAndLanguagecode(Register register, Languagecode languagecode) throws Exception {
        //Checking parameters
        if (languagecode == null || register == null) {
            throw new Exception("The languagecode or register passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByRegisterAndLanguagecode");
        q.setParameter("language", languagecode);
        q.setParameter("register", register);
        return (Localization) q.getSingleResult();
    }

    public Localization findByRegistryAndLanguagecode(Registry registry, Languagecode languagecode) throws Exception {
        //Checking parameters
        if (languagecode == null || registry == null) {
            throw new Exception("The languagecode or registry passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByRegistryAndLanguagecode");
        q.setParameter("language", languagecode);
        q.setParameter("registry", registry);
        return (Localization) q.getSingleResult();
    }

    public Localization findByCustomattributevalueAndLanguagecode(Customattributevalue customattributevalue, Languagecode languagecode) throws Exception {
        //Checking parameters
        if (languagecode == null || customattributevalue == null) {
            throw new Exception("The languagecode or customattributevalue passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByCustomattributevalueAndLanguagecode");
        q.setParameter("language", languagecode);
        q.setParameter("customattributevalue", customattributevalue);
        return (Localization) q.getSingleResult();
    }

    public List<Localization> findRegistryLocalizationAfterDate(Registry registry, Date date) throws Exception {
        //Checking parameters
        if (registry == null || date == null) {
            throw new Exception("The registry or date passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByRegistryAndAfterDate");
        q.setParameter("registry", registry);
        q.setParameter("date", date);
        return (List<Localization>) q.getResultList();
    }

    public List<Localization> findRegistryLocalization(Registry registry) throws Exception {
        //Checking parameters
        if (registry == null) {
            throw new Exception("The registry passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByRegistry");
        q.setParameter("registry", registry);
        return (List<Localization>) q.getResultList();
    }

    public List<Localization> findRegisterLocalizationAfterDate(Register register, Date date) throws Exception {
        //Checking parameters
        if (register == null || date == null) {
            throw new Exception("The register or date passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByRegisterAndAfterDate");
        q.setParameter("register", register);
        q.setParameter("date", date);
        return (List<Localization>) q.getResultList();
    }

    public List<Localization> findRegisterLocalization(Register register) throws Exception {
        //Checking parameters
        if (register == null) {
            throw new Exception("The register passed by parameter is null.");
        }

        //Preparing query
        Query q = em.createNamedQuery("Localization.findByRegister");
        q.setParameter("register", register);
        return (List<Localization>) q.getResultList();
    }

    public List<Localization> findByItem(Item item) throws Exception {
        //Checking parameters
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Localization.findByItem");
        q.setParameter("item", item);

        return (List<Localization>) q.getResultList();
    }

    public List<Localization> findByCustomattributevalue(Customattributevalue cav) throws Exception {
        //Checking parameters
        if (cav == null) {
            throw new Exception("The Customattributevalue passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Localization.findByCustomattributevalue");
        q.setParameter("customattributevalue", cav);

        return (List<Localization>) q.getResultList();
    }

    /**
     * Insert the specified localizations in the database. Returns true if the
     * operation succeed.
     *
     * @param lo the localization object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertLocalization(Localization lo) throws Exception {
        //Checking parameters
        if (lo == null) {
            throw new Exception("The Localization passed by parameter is null.");
        }

        if (lo.getDatecreation() == null) {
            lo.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(lo);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(lo);

        return true;
    }

    /**
     * Update the localization specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param lo the updated localization (this method update the localization
     * on the db with the localization passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateLocalization(Localization lo) throws Exception {
        //Checking parameters
        if (lo == null) {
            throw new Exception("The Localization passed by parameter is null.");
        }

        if (lo.getDatelastupdate() == null) {
            lo.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(lo, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(lo);

        return true;
    }

    /**
     * Delete from the database the localization specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param lo the localization do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteLocalization(Localization lo) throws Exception {
        //Checking parameters
        if (lo == null) {
            throw new Exception("The Localization passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(lo);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Localization lo) throws Exception {
        if (lo.getUuid() == null || lo.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (lo.getLanguage() == null) {
            throw new Exception("Language field not available.");
        }

        if (lo.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Localization lo, boolean isUpdate) throws Exception {
        if (lo.getUuid() == null || lo.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (lo.getLanguage() == null) {
            throw new Exception("Language field not available.");
        }

        if (lo.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && lo.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }
}
