/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Registry;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class RegistryMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public RegistryMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the registry with the specified uuid.
     *
     * @param uuid the unique id of the registry
     * @return the registry with the specified uuid
     * @throws Exception
     */
    public Registry getRegistryByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Registry.findByUuid");
        q.setParameter("uuid", uuid);
        return (Registry) q.getSingleResult();
    }

    public List<Registry> findRegistryAfterDate(Date latestStaticizerDatestart) throws Exception {
        //Checking parameters
        if (latestStaticizerDatestart == null) {
            throw new Exception("The latestStaticizerDatestart passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Registry.findRegistryAfterDate");
        q.setParameter("date", latestStaticizerDatestart);
        return (List<Registry>) q.getResultList();
    }

    public List<Registry> findAll() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Registry.findAll");
        return (List<Registry>) q.getResultList();
    }

    /**
     * Insert the specified registrys in the database. Returns true if the
     * operation succeed.
     *
     * @param re the registry object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertRegistry(Registry re) throws Exception {
        //Checking parameters
        if (re == null) {
            throw new Exception("The Registry passed by parameter is null.");
        }

        if (re.getDatecreation() == null) {
            re.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(re);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(re);

        return true;
    }

    /**
     * Update the registry specified by parameter. Returns true if the operation
     * succeed.
     *
     * @param re the updated registry (this method update the registry on the db
     * with the registry passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateRegistry(Registry re) throws Exception {
        //Checking parameters
        if (re == null) {
            throw new Exception("The Registry passed by parameter is null.");
        }

        if (re.getDatelastupdate() == null) {
            re.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(re, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(re);

        return true;
    }

    /**
     * Delete from the database the registry specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param re the registry do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteRegistry(Registry re) throws Exception {
        //Checking parameters
        if (re == null) {
            throw new Exception("The Registry passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(re);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Registry re) throws Exception {
        if (re.getUuid() == null || re.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (re.getBaseuri() == null) {
            throw new Exception("Baseuri field not available.");
        }

        if (re.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        if (re.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Registry re, boolean isUpdate) throws Exception {
        if (re.getUuid() == null || re.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (re.getBaseuri() == null) {
            throw new Exception("Baseuri field not available.");
        }

        if (re.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        if (re.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && re.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }
}
