/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Staticizer;
import eu.europa.ec.re3gistrycommon.model.Procedurestatus;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class StaticizerMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public StaticizerMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns all staticizer
     *
     * @return all the staticizer
     * @throws Exception
     */
    public List<Staticizer> getAllStaticizerOrderDescByDatestart() throws Exception {
        Query q = this.em.createNamedQuery("Staticizer.findAllOrderDescByDateStart");
        return (List<Staticizer>) q.getResultList();
    }

    /**
     * Returns List<Staticizer> completed order by date start
     *
     * @throws Exception
     */
    public List<Staticizer> findLastCompletedOrderDescByDateStart() throws Exception {
        Query qs = this.em.createNamedQuery("Procedurestatus.findByUuid");
        qs.setParameter("uuid", "3");
        Procedurestatus statusCompleted = (Procedurestatus) qs.getSingleResult();

        Query q = this.em.createNamedQuery("Staticizer.findLastCompletedOrderDescByDateStart");
        q.setParameter("procedurestatus", statusCompleted);
        return (List<Staticizer>) q.getResultList();
    }

    /**
     * Returns List<Staticizer> in progress order by date start
     *
     * @throws Exception
     */
    public List<Staticizer> findInProgressOrderDescByDateStart() throws Exception {
        Query qs = this.em.createNamedQuery("Procedurestatus.findByUuid");
        qs.setParameter("uuid", "1");
        Procedurestatus statusInProgress = (Procedurestatus) qs.getSingleResult();

        Query q = this.em.createNamedQuery("Staticizer.findInProgressOrderDescByDateStart");
        q.setParameter("procedurestatus", statusInProgress);
        return (List<Staticizer>) q.getResultList();
    }

    /**
     * Returns List<Staticizer> queued order by date start
     *
     * @throws Exception
     */
    public List<Staticizer> findQueuedOrderAscByDateStart() throws Exception {
        Query qs = this.em.createNamedQuery("Procedurestatus.findByUuid");
        qs.setParameter("uuid", "5");
        Procedurestatus statusQueue = (Procedurestatus) qs.getSingleResult();

        Query q = this.em.createNamedQuery("Staticizer.findQueuedOrderAscByDateStart");
        q.setParameter("procedurestatus", statusQueue);
        return (List<Staticizer>) q.getResultList();
    }

    /**
     * @return a list of Staticizer completed, after date ordered ASC 
     * @param date 
     * @throws Exception
     */
    public List<Staticizer> findAllCompletedAfterDateOrderDescByAndDateStart(Date date) throws Exception {
        Query qs = this.em.createNamedQuery("Procedurestatus.findByUuid");
        qs.setParameter("uuid", "3");
        Procedurestatus statusCompleted = (Procedurestatus) qs.getSingleResult();

        Query q = this.em.createNamedQuery("Staticizer.findAllCompletedAfterDateOrderDescByAndDateStart");
        q.setParameter("procedurestatus", statusCompleted);
        q.setParameter("date", date);
        return (List<Staticizer>) q.getResultList();
    }

    /**
     * @return the last completed staticizer
     * 
     * @throws Exception
     */
    public Staticizer getLastCompletedStaticizer() throws Exception {
        List<Staticizer> staticizerList = findLastCompletedOrderDescByDateStart();
        return staticizerList.get(0);
    }

    /**
     * Returns the staticizer with the specified uuid.
     *
     * @param uuid the unique id of the staticizer
     * @return the staticizer with the specified uuid
     * @throws Exception
     */
    public Staticizer getStaticizerByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Staticizer.findByUuid");
        q.setParameter("uuid", uuid);
        return (Staticizer) q.getSingleResult();
    }

    /**
     * Insert the specified staticizers in the database. Returns true if the
     * operation succeed.
     *
     * @param st the staticizer object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertStaticizer(Staticizer st) throws Exception {
        //Checking parameters
        if (st == null) {
            throw new Exception("The Staticizer passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(st);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(st);

        return true;
    }

    /**
     * Update the staticizer specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param st the updated staticizer (this method update the staticizer on
     * the db with the staticizer passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateStaticizer(Staticizer st) throws Exception {
        //Checking parameters
        if (st == null) {
            throw new Exception("The Staticizer passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(st, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(st);

        return true;
    }

    /**
     * Delete from the database the staticizer specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param st the staticizer do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteStaticizer(Staticizer st) throws Exception {
        //Checking parameters
        if (st == null) {
            throw new Exception("The Staticizer passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(st);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Staticizer st) throws Exception {
        if (st.getUuid() == null || st.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (st.getDatestart() == null) {
            throw new Exception("Datestart field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Staticizer st, boolean isUpdate) throws Exception {
        if (st.getUuid() == null || st.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (st.getDatestart() == null) {
            throw new Exception("Datestart field not available.");
        }
        if (isUpdate && st.getDateend() == null) {
            throw new Exception("Dateend field not available.");
        }

        return true;
    }
}