/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Status;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class StatusMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public StatusMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the status with the specified uuid.
     *
     * @param uuid the unique id of the status
     * @return the status with the specified uuid
     * @throws Exception
     */
    public Status getStatusByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Status.findByUuid");
        q.setParameter("uuid", uuid);
        return (Status) q.getSingleResult();
    }

    /**
     * Returns the status with the specified uriname.
     *
     * @param uriname the uriname of the status
     * @return the status with the specified uriname
     * @throws Exception
     */
    public Status getStatusByUriname(String uriname) throws Exception {
        //Checking parameters
        if (uriname == null) {
            throw new Exception("The uriname passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Status.findByUriname");
        q.setParameter("uriname", uriname);
        return (Status) q.getSingleResult();
    }

    /**
     * Insert the specified status in the database. Returns true if the
     * operation succeed.
     *
     * @param st the status object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertStatus(Status st) throws Exception {
        //Checking parameters
        if (st == null) {
            throw new Exception("The Status passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(st);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(st);

        return true;
    }

    /**
     * Update the status specified by parameter. Returns true if the operation
     * succeed.
     *
     * @param st the updated status (this method update the status on the db
     * with the status passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateStatus(Status st) throws Exception {
        //Checking parameters
        if (st == null) {
            throw new Exception("The Status passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(st, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(st);

        return true;
    }

    /**
     * Delete from the database the status specified by parameter. Returns true
     * if the operation succeed.
     *
     * @param st the status do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteStatus(Status st) throws Exception {
        //Checking parameters
        if (st == null) {
            throw new Exception("The Status passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(st);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Status st) throws Exception {
        if (st.getUuid() == null || st.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (st.getBaseuri() == null) {
            throw new Exception("Baseuri field not available.");
        }

        if (st.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Status st, boolean isUpdate) throws Exception {
        if (st.getUuid() == null || st.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (st.getBaseuri() == null) {
            throw new Exception("Baseuri field not available.");
        }

        if (st.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        return true;
    }
}
