/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers;

import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.utils.Mail;
import eu.europa.ec.re3gistrycommon.utils.PersistenceFactory;
import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.Clarification;
import eu.europa.ec.re3gistrydata.beans.Field;
import eu.europa.ec.re3gistrydata.beans.Invalidation;
import eu.europa.ec.re3gistrydata.beans.Report;
import eu.europa.ec.re3gistrydata.beans.Retirement;
import eu.europa.ec.re3gistrydata.beans.Supersession;
import eu.europa.ec.re3gistrydata.constants.Constants;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import org.apache.log4j.Logger;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.IOUtils;

public class DataMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    //Variables
    private HttpServletRequest request;
    private Properties properties;
    private ReportMgr reportMgr;
    private FormatMgr formatMgr;
    private StoreMgr storeMgr;
    private AnalyzerMgr analyzerMgr;

    //Configuration variable
    private String tempDataFolder;
    private String thisProcedureFolder;

    //Hashmaps
    private LinkedHashMap<String, Addition> additions;
    private LinkedHashMap<String, Clarification> clarifications;
    private LinkedHashMap<String, Supersession> supersessions;
    private LinkedHashMap<String, Invalidation> invalidations;
    private LinkedHashMap<String, Retirement> retirements;

    private List<Field> customAttributeDescriptors;

    //Parameters from webpage
    private String procedureDescription;
    private String fwarningsFlag;

    //Localization
    private ResourceBundle localizationres;

    /**
     * Class initializer.
     *
     * @param request
     * @param reportMgr
     * @throws Exception
     */
    public DataMgr(HttpServletRequest request, ReportMgr reportMgr) throws Exception {
        try {
            log.info("# Initializing data manager ...");

            //Setting the request parameter
            if (request == null) {
                throw new Exception("Error during DataMgr initialization: request parameter is null.");
            }
            this.request = request;

            //Initializing the report manager
            if (reportMgr == null) {
                throw new Exception("Error during DataMgr initialization: reportMgr parameter is null.");
            }
            this.reportMgr = reportMgr;

            //Loading module properties
            this.properties = PropertyMgr.getProperties();

            //Setting the temp data folder
            this.setTempFolder(request);

            //Initializing the data hashmap
            additions = new LinkedHashMap<String, Addition>();
            clarifications = new LinkedHashMap<String, Clarification>();
            supersessions = new LinkedHashMap<String, Supersession>();
            invalidations = new LinkedHashMap<String, Invalidation>();
            retirements = new LinkedHashMap<String, Retirement>();

            //Initializing the customAttributeDescriptors
            customAttributeDescriptors = new ArrayList<Field>();

            //Initializing the format checker
            this.formatMgr = new FormatMgr(reportMgr, this);

            //Initializing the StoreMgr
            this.analyzerMgr = new AnalyzerMgr(reportMgr, this);

            //Initializing the StoreMgr
            this.storeMgr = new StoreMgr(reportMgr, this);

            //Initializing the system localization
            HttpSession s = this.request.getSession();
            String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
            if (currentLanguage == null) {
                currentLanguage = this.request.getLocale().toString();
            }
            this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

            log.info("# Data manager initialized.");
        } catch (Exception e) {
            throw new Exception("Error during DataMgr initialization: " + e.getMessage());
        }
    }

    /**
     * Start the data procedure
     *
     * @throws Exception
     */
    public void startProcedure() throws Exception {
        EntityManager em = null;
        try {
            //Unzipping files & getting parameters from webpage
            this.prepareFiles();

            //Initializing the entity mgr;
            em = PersistenceFactory.getEntityManagerFactory().createEntityManager();

            //Starting format check
            this.checkFormat(em);

            //If thete are no problem in the format
            if (reportMgr.size() <= 0) {
                this.analyzerMgr.analyzeData(em);
            } else {
                request.setAttribute(Constants.ERROR_TYPE, Constants.ERROR_KEY_FORMAT);

                //Sending mail in case of error
                Mail m = new Mail();
                String subject = localizationres.getString("export.mail.subject");
                String body = localizationres.getString("export.mail.body.errorwarning").replace("{0}", Constants.ERROR_KEY_FORMAT);
                //Creating attachment                
                File tempFile = this.createAttachmentFile();
                m.sendbyMail(request, tempFile, subject, body);

                throw new Exception();
            }

            //Seraching for errors
            int errors = 0;
            for (Report tmp : reportMgr.getReports()) {
                if (tmp.getType().equals(Constants.REPORT_TYPE_ERROR)) {
                    errors++;
                }
            }

            if (reportMgr.size() <= 0 || (reportMgr.size() > 0 && errors == 0 && this.fwarningsFlag != null && this.fwarningsFlag.equals(Constants.CHECKBOX_ON_VALUE))) {
                this.storeMgr.storeData(this.procedureDescription, em);
                request.setAttribute(Constants.FORM_FIELD_FWARNINGS, this.fwarningsFlag);

                if (reportMgr.size() > 0) {
                    request.setAttribute(Constants.ERROR_TYPE, Constants.ERROR_KEY_STORE);

                    //Sending mail in case of error
                    Mail m = new Mail();
                    String subject = localizationres.getString("export.mail.subject");
                    String body = localizationres.getString("export.mail.body.errorwarning").replace("{0}", Constants.ERROR_KEY_STORE);
                    //Creating attachment                
                    File tempFile = this.createAttachmentFile();
                    m.sendbyMail(request, tempFile, subject, body);
                } else {
                    //Sending mail in case of success
                    Mail m = new Mail();
                    String subject = localizationres.getString("export.mail.subject");
                    String body = localizationres.getString("export.mail.body.success");
                    m.sendbyMail(request, null, subject, body);
                }
            } else {
                request.setAttribute(Constants.ERROR_TYPE, Constants.ERROR_KEY_ANALYZER);
                //Sending mail in case of error
                Mail m = new Mail();
                String subject = localizationres.getString("export.mail.subject");
                String body = localizationres.getString("export.mail.body.errorwarning").replace("{0}", Constants.ERROR_KEY_ANALYZER);
                //Creating attachment                
                File tempFile = this.createAttachmentFile();
                m.sendbyMail(request, tempFile, subject, body);
            }
        } catch (Exception e) {
            throw e;
        } finally {
            if (em != null && em.isOpen()) {
                em.close();
            }
        }
    }

    /**
     * Check if there is a custom temp folder.
     */
    private void setTempFolder(HttpServletRequest request) {

        log.info("# Setting up the data temp folder");
        this.tempDataFolder = this.request.getSession().getServletContext().getRealPath(Constants.DEFAULT_TEMP_FOLDER_NAME);

        String customTempDataFolder = properties.getProperty(Constants.KEY_CUSTOM_TEMP_FOLDER);
        if (customTempDataFolder != null && customTempDataFolder.length() > 0) {
            this.tempDataFolder = customTempDataFolder;
            log.info("# Custom data temp folder found: " + this.tempDataFolder);
        }

        //creating custom folder for this procedure
        this.thisProcedureFolder = String.valueOf(System.currentTimeMillis());
        this.tempDataFolder += File.separator + this.thisProcedureFolder;

        //Setting the current data file temp folder for next processing
        request.getSession().setAttribute(Constants.TEMPDATAFILE_KEY, this.tempDataFolder);

        File f = new File(this.tempDataFolder);
        if (!f.exists()) {
            f.mkdirs();
        }

    }

    /**
     * Unzip procedure.
     */
    private void prepareFiles() throws Exception {
        log.info("# Statring the data file unzip.");

        List<FileItem> fl = new ServletFileUpload(new DiskFileItemFactory()).parseRequest(this.request);
        FileItem dataFile = null;

        for (FileItem item : fl) {
            if (!item.isFormField()) {
                dataFile = item;
            } else {
                if (item.getFieldName().equals(Constants.FORM_FIELD_COMMENT)) {
                    this.procedureDescription = item.getString();
                }
                if (item.getFieldName().equals(Constants.FORM_FIELD_FWARNINGS)) {
                    this.fwarningsFlag = item.getString();
                }
            }
        }

        if (dataFile == null || dataFile.getSize() == 0) {
            throw new Exception(this.localizationres.getString("error.datamgr.unzip.importfile"));
        }

        //Gettingg the current data file temp folder
        String folderName = (String) request.getSession().getAttribute(Constants.TEMPDATAFILE_KEY);

        //Storing the zip file
        File file = new File(folderName + Constants.ZIP_FILE_EXTENSION);
        if (!file.exists()) {
            file.createNewFile();
        }
        dataFile.write(file);

        log.info("# Unzipping data file ...");
        if (!ZipMgr.unzipFile(this.tempDataFolder, file)) {
            throw new Exception(this.localizationres.getString("error.datamgr.unzip.unziperror"));
        }
        log.info("# File succesfully unzipped.");

    }

    /**
     * This method prepares the check format procedure and launch it for each
     * input file
     *
     */
    private void checkFormat(EntityManager em) throws Exception {
        log.info("# Starting format check.");
        File dataFolder = new File(this.tempDataFolder);
        //Checking the data temp folder
        if (!dataFolder.exists()) {
            throw new Exception(this.localizationres.getString("error.datamgr.checkformatinit.notempfolder"));
        }
        if (!dataFolder.isDirectory()) {
            throw new Exception(this.localizationres.getString("error.datamgr.checkformatinit.tempfoldernotdir"));
        }

        //Getting data flow order
        ItemclassMgr itemclassMgr = new ItemclassMgr(em);
        List<Itemclass> itemclasses = itemclassMgr.getAllItemclassOrderedByDataprocedureorder();
        //Process elements in order
        for (int i = 0; i < itemclasses.size(); i++) {
            Itemclass tmp = itemclasses.get(i);
            //Getting itemclass folder
            File itemclassFolder = new File(dataFolder + File.separator + tmp.getUriname());

            if (!itemclassFolder.exists()) {
                //reportMgr.addReport(Constants.REPORT_CONTEXT_DATAMGR, Constants.REPORT_ACTION_DATAMGR_CHECKFILES, "Check format initializar,  the zip does not contain the folder for " + tmp.getUriname() + ".", Constants.REPORT_TYPE_ERROR, -1);
                //log.error("@@ Check format initializer, the zip does not contain the folder for " + tmp.getUriname() + ".");
            } else {

                File[] actionFiles = itemclassFolder.listFiles(new FilenameFilter() {
                    @Override
                    public boolean accept(File dir, String filename) {
                        return filename.endsWith(Constants.ACTION_FILE_EXTENSION);
                    }
                });
                //Processing each file
                for (File actionFile : actionFiles) {
                    this.formatMgr.checkFormat(actionFile, itemclassFolder.getName());
                }
            }
        }
        log.info("# Format check completed.");
    }

    /**
     * This method returns the LinkedHashMap containing a data object for each
     * row contained in the CSV file processed by the FormatChecker
     *
     * @return LinkedHashMap containing data from the action files.
     */
    public LinkedHashMap<String, Addition> getAdditions() {
        return additions;
    }

    /**
     * This method returns the LinkedHashMap containing a data object for each
     * row contained in the CSV file processed by the FormatChecker
     *
     * @return LinkedHashMap containing data from the action files.
     */
    public LinkedHashMap<String, Clarification> getClarification() {
        return clarifications;
    }

    /**
     * This method returns the LinkedHashMap containing a data object for each
     * row contained in the CSV file processed by the FormatChecker
     *
     * @return LinkedHashMap containing data from the action files.
     */
    public LinkedHashMap<String, Supersession> getSupersession() {
        return supersessions;
    }

    /**
     * This method returns the LinkedHashMap containing a data object for each
     * row contained in the CSV file processed by the FormatChecker
     *
     * @return LinkedHashMap containing data from the action files.
     */
    public LinkedHashMap<String, Invalidation> getInvalidation() {
        return invalidations;
    }

    /**
     * This method returns the LinkedHashMap containing a data object for each
     * row contained in the CSV file processed by the FormatChecker
     *
     * @return LinkedHashMap containing data from the action files.
     */
    public LinkedHashMap<String, Retirement> getRetirement() {
        return retirements;
    }

    /**
     * This method returns a List containing customAttributeDescriptors
     * processed in the Format checker (while checking the CSV header)
     *
     * @return LinkedHashMap containing the customAttributeDescriptors.
     */
    public List<Field> getCustomAttributeDescriptors() {
        return customAttributeDescriptors;
    }

    public String getTempDataFolder() {
        return tempDataFolder;
    }

    public String getProcedureDataFolder() {
        return this.thisProcedureFolder;
    }

    public HttpServletRequest getRequest() {
        return request;
    }

    private File createAttachmentFile() throws Exception {
        String csvSeparator = Constants.MAILATTACH_CSV_SEPARATOR;
        String outfile = "";
        for (Report report : this.reportMgr.getReports()) {
            outfile += "\"" + report.getContext() + "\"";
            outfile += csvSeparator;
            outfile += "\"" + report.getAction() + "\"";
            outfile += csvSeparator;
            outfile += "\"" + Integer.toString(report.getLine()) + "\"";
            outfile += csvSeparator;
            outfile += "\"" + report.getMessage() + "\"";
            outfile += "\r\n";
        }
        //Creating the temp file
        InputStream is = new ByteArrayInputStream(outfile.getBytes(Constants.FILE_ENCODING));
        final File tempFile = File.createTempFile(Constants.KEY_EXPORT_MAIL_TMPFILE_PREFIX, Constants.KEY_EXPORT_MAIL_TMPFILE_SUFFIX);
        tempFile.deleteOnExit();
        FileOutputStream out = new FileOutputStream(tempFile);
        IOUtils.copy(is, out);
        return tempFile;
    }
}
