/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers;

import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.Clarification;
import eu.europa.ec.re3gistrydata.beans.CsvHeader;
import eu.europa.ec.re3gistrydata.beans.CustomattributeDescriptor;
import eu.europa.ec.re3gistrydata.beans.Field;
import eu.europa.ec.re3gistrydata.beans.Invalidation;
import eu.europa.ec.re3gistrydata.beans.Retirement;
import eu.europa.ec.re3gistrydata.beans.Supersession;
import eu.europa.ec.re3gistrydata.constants.Constants;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class FormatMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    //Common variables
    private final Properties properties;
    private final ReportMgr reportMgr;

    //Variable for properties
    private final String csvDelimiter;
    private final String customAttributePrefix;
    private final String customAttributeDelimiter;
    private final String customAttributePropertyOpen;
    private final String customAttributePropertyClose;
    private final String mandatoryfieldsDelimiter;
    private final String headerFieldsKey;
    private final String mandatoryFieldsKey;

    //Variables for common fields count
    private final int commonFieldAddition;
    private final int commonFieldClarification;
    private final int commonFieldSupersession;
    private final int commonFieldInvalidation;
    private final int commonFieldRetirement;

    //Variables for mandatoryFields check
    private final int[] mandatoryFieldAddition;
    private final int[] mandatoryFieldClarification;
    private final int[] mandatoryFieldSupersession;
    private final int[] mandatoryFieldInvalidation;
    private final int[] mandatoryFieldRetirement;

    //Hashmaps
    private final LinkedHashMap<String, Addition> additions;
    private final LinkedHashMap<String, Clarification> clarifications;
    private final LinkedHashMap<String, Supersession> supersessions;
    private final LinkedHashMap<String, Invalidation> invalidations;
    private final LinkedHashMap<String, Retirement> retirements;

    //Custom attributes list
    private final List<Field> customAttributeDescriptors;

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param reportMgr
     * @param dataMgr
     * @throws Exception
     */
    public FormatMgr(ReportMgr reportMgr, DataMgr dataMgr) throws Exception {
        log.info("# Initializing the Format check ...");

        this.properties = PropertyMgr.getProperties();
        this.reportMgr = reportMgr;

        //Reading values from properties
        this.csvDelimiter = this.properties.getProperty(Constants.KEY_CSV_DELIMITER, "|");
        this.customAttributePrefix = this.properties.getProperty(Constants.KEY_CUSTOMATTRIBUTE_PREFIX, "*");
        this.customAttributePropertyOpen = this.properties.getProperty(Constants.KEY_CUSTOM_ATTRIBUTE_OPEN, ",");
        this.customAttributePropertyClose = this.properties.getProperty(Constants.KEY_CUSTOM_ATTRIBUTE_CLOSE, ",");
        this.customAttributeDelimiter = this.properties.getProperty(Constants.KEY_CUSTOM_ATTRIBUTE_DELIMITER, ",");
        this.mandatoryfieldsDelimiter = this.properties.getProperty(Constants.KEY_MANDATORY_FIELD_DELIMITER, ".");

        this.headerFieldsKey = this.properties.getProperty(Constants.KEY_COMMONFIELDS_NUMBER, "data.commonfields.");
        this.mandatoryFieldsKey = this.properties.getProperty(Constants.KEY_MANDATORY_FIELDS, "data.mantatoryfields.");

        //Getting properties for common fields for each actions
        this.commonFieldAddition = Integer.parseInt(this.properties.getProperty(this.headerFieldsKey + Constants.ACTION_ADDITION, null));
        this.commonFieldClarification = Integer.parseInt(this.properties.getProperty(this.headerFieldsKey + Constants.ACTION_CLARIFICATION, null));
        this.commonFieldSupersession = Integer.parseInt(this.properties.getProperty(this.headerFieldsKey + Constants.ACTION_SUPERSESSION, null));
        this.commonFieldInvalidation = Integer.parseInt(this.properties.getProperty(this.headerFieldsKey + Constants.ACTION_INVALIDATION, null));
        this.commonFieldRetirement = Integer.parseInt(this.properties.getProperty(this.headerFieldsKey + Constants.ACTION_RETIREMENT, null));

        //Getting properties for mandatory fields for each actions
        this.mandatoryFieldAddition = this.requireFieldInitializer(Constants.ACTION_ADDITION);
        this.mandatoryFieldClarification = this.requireFieldInitializer(Constants.ACTION_CLARIFICATION);
        this.mandatoryFieldSupersession = this.requireFieldInitializer(Constants.ACTION_SUPERSESSION);
        this.mandatoryFieldInvalidation = this.requireFieldInitializer(Constants.ACTION_INVALIDATION);
        this.mandatoryFieldRetirement = this.requireFieldInitializer(Constants.ACTION_RETIREMENT);

        //Initializing hashmap
        additions = dataMgr.getAdditions();
        clarifications = dataMgr.getClarification();
        supersessions = dataMgr.getSupersession();
        invalidations = dataMgr.getInvalidation();
        retirements = dataMgr.getRetirement();

        customAttributeDescriptors = dataMgr.getCustomAttributeDescriptors();

        //Initializing the system localization
        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

        log.info("# Format check initialized.");
    }

    /**
     * This method start the format check for the file passed by parameter
     *
     * @param inFile
     * @param itemClass
     * @throws Exception
     */
    public void checkFormat(File inFile, String itemClass) throws Exception {

        //Getting the action for this file
        String action = inFile.getName().replace(Constants.ACTION_FILE_EXTENSION, "");

        //Getting file content
        InputStream filecontent = new FileInputStream(inFile);
        BufferedReader in = new BufferedReader(new InputStreamReader(filecontent, Constants.FILE_ENCODING));
        log.info("# Format check for " + itemClass + ", action >" + action);

        //Itinializing variables
        String text;

        //Initializing counters
        int line = 1;

        CsvHeader csvHeader = null;
        while ((text = in.readLine()) != null) {

            //Splitting current row
            String[] tempData = text.split(this.csvDelimiter, -1);

            //If the current row is the header row, check the number of fields 
            //and store the customattributedescriptor in the list
            if (line == 1) {
                try {
                    log.info("#");
                    log.info("# Checking the CSV header.");
                    csvHeader = this.checkCsvHeader(tempData, action, itemClass);
                    log.info("# CSV header check complete.");
                    log.info("#");
                } catch (Exception e) {
                    reportMgr.addReport(itemClass, action, e.getMessage(), Constants.REPORT_TYPE_ERROR, line);
                    log.error("@@ Format check, " + e.getMessage());
                    break;
                }
                //Else check the data row and store data in the data hashmaps    
            } else {
                try {
                    log.debug("# Checking row " + line + ".");
                    //Checking current row
                    this.checkRow(tempData, csvHeader, action, itemClass);
                } catch (Exception e) {
                    reportMgr.addReport(itemClass, action, e.getMessage(), Constants.REPORT_TYPE_ERROR, line);
                    log.error("@@ Format check, " + e.getMessage());
                }
            }
            line++;
        }

        in.close();
        filecontent.close();
    }

    /**
     * This method check the csv header passed by parameter and returns the csv
     * header as an object
     *
     * @param [] data
     * @param action
     * @param itemClass
     * @throws Exception
     */
    private CsvHeader checkCsvHeader(String[] data, String action, String itemClass) throws Exception {
        //Count if the standard csv fields are available (the first fields are the 
        //standard fields in the CSV structure. They have to be provided even if
        //they are null)

        //Initializing variables
        int standardFieldCount = 0;
        CsvHeader csvHeader = new CsvHeader();

        //Checking the field is a custom attribute
        int x = 0;
        for (String element : data) {
            if (element != null) {
                element = element.trim();
                if (element.length() > 0 && !element.startsWith(this.customAttributePrefix)) {
                    //If the current element is not a custom attribute                    
                    standardFieldCount++;

                    //Loading the mandatory fields list
                    int[] check = null;
                    if (action.equals(Constants.ACTION_ADDITION)) {
                        check = this.mandatoryFieldAddition;
                    } else if (action.equals(Constants.ACTION_CLARIFICATION)) {
                        check = this.mandatoryFieldClarification;
                    } else if (action.equals(Constants.ACTION_SUPERSESSION)) {
                        check = this.mandatoryFieldSupersession;
                    } else if (action.equals(Constants.ACTION_INVALIDATION)) {
                        check = this.mandatoryFieldInvalidation;
                    } else if (action.equals(Constants.ACTION_RETIREMENT)) {
                        check = this.mandatoryFieldRetirement;
                    }

                    if (check == null) {
                        throw new Exception(this.localizationres.getString("error.formatmgr.csvheader.norequiredfields"));
                    }
                    boolean required = false;
                    //Checking if the current field is required
                    for (int i = 0; i < check.length; i++) {
                        if (x == check[i]) {
                            required = true;
                        }
                    }

                    //Creating the field and adding it to the csv header
                    Field f = new Field(x, element, itemClass, required);
                    csvHeader.addField(x, f);
                    x++;

                } else if (element.length() > 0 && element.startsWith(this.customAttributePrefix)) {
                    //If the element is a custom attribute
                    log.info("# Custom attribute found in the CSV header");
                    String caName = element.substring(1, element.indexOf(this.customAttributePropertyOpen));
                    String tmp = element.substring(element.indexOf(this.customAttributePropertyOpen) + 1, element.indexOf(this.customAttributePropertyClose));
                    String tmps[] = tmp.split(this.customAttributeDelimiter);
                    if (tmps.length < 4) {
                        throw new Exception(this.localizationres.getString("error.formatmgr.csvheader.cadesc"));
                    }

                    boolean required = tmps[0].equals("t");
                    boolean multivalued = tmps[1].equals("t");
                    boolean coded = tmps[2].equals("t");
                    boolean foreignkey = tmps[3].equals("t");

                    //if the action is clarification, the CA shall be specified only if it has to be modified
                    if (action.equals(Constants.ACTION_CLARIFICATION)) {
                        required = false;
                    }

                    //Creating the custom attribute field and adding it to the csv header
                    Field f = new Field(x, caName, itemClass, required, true, multivalued, coded, foreignkey);
                    csvHeader.addField(x, f);

                    //Adding the custom attribute descriptor to the custom attribute descriptor list
                    customAttributeDescriptors.add(f);
                    x++;
                }
            }
        }

        //Cheching if all the common fields are available in the header (all field except the possible custom attributes)
        int requiredStandardFieldsCount = 0;
        if (action.equals(Constants.ACTION_ADDITION)) {
            requiredStandardFieldsCount = this.commonFieldAddition;
        } else if (action.equals(Constants.ACTION_CLARIFICATION)) {
            requiredStandardFieldsCount = this.commonFieldClarification;
        } else if (action.equals(Constants.ACTION_SUPERSESSION)) {
            requiredStandardFieldsCount = this.commonFieldSupersession;
        } else if (action.equals(Constants.ACTION_INVALIDATION)) {
            requiredStandardFieldsCount = this.commonFieldInvalidation;
        } else if (action.equals(Constants.ACTION_RETIREMENT)) {
            requiredStandardFieldsCount = this.commonFieldRetirement;
        }

        //Report an error if the standard fields number are not as configured in the properties file
        if (standardFieldCount != requiredStandardFieldsCount) {
            String tmp = this.localizationres.getString("error.formatmgr.csvheader.fieldmismatch").replace("{0}", Integer.toString(requiredStandardFieldsCount)).replace("{1}", Integer.toString(standardFieldCount));
            throw new Exception(tmp);
        }

        return csvHeader;
    }

    /**
     * This method check the CSV row header as an object
     *
     * @param [] data
     * @param csvHeader
     * @param action
     * @param itemClass
     * @throws Exception
     */
    private void checkRow(String[] data, CsvHeader csvHeader, String action, String itemClass) throws Exception {

        //Getting the csw header field number
        int fields = csvHeader.countFields();

        //Check if this row has all the fields specified in the csw header
        if (data.length != fields) {
            //"Check row: fields mismatch. CSV header fields: " + fields + ", this row fields: " + data.length + "."
            String tmp = this.localizationres.getString("error.formatmgr.csvheader.fieldmismatchdata").replace("{0}", Integer.toString(fields)).replace("{1}", Integer.toString(data.length));
            throw new Exception(tmp);
        }

        //Get mantatory field list
        int[] check = null;
        if (action.equals(Constants.ACTION_ADDITION)) {
            check = this.mandatoryFieldAddition;
        } else if (action.equals(Constants.ACTION_CLARIFICATION)) {
            check = this.mandatoryFieldClarification;
        } else if (action.equals(Constants.ACTION_SUPERSESSION)) {
            check = this.mandatoryFieldSupersession;
        } else if (action.equals(Constants.ACTION_INVALIDATION)) {
            check = this.mandatoryFieldInvalidation;
        } else if (action.equals(Constants.ACTION_RETIREMENT)) {
            check = this.mandatoryFieldRetirement;
        }

        if (check == null) {
            throw new Exception(this.localizationres.getString("error.formatmgr.row.nomandatoryprop"));
        }

        //Check if the required field is available
        String errors = "";
        for (int i = 0; i < data.length; i++) {
            Field tmp = csvHeader.getFields().get(i);
            if (tmp.isRequired() && (data[i] == null || data[i].equals(""))) {
                errors += tmp.getName() + ", ";
            }
        }

        //If some fields are not available, store the error
        if (errors.length() > 0) {
            errors = errors.substring(0, errors.length() - 2);
            throw new Exception(this.localizationres.getString("error.formatmgr.row.nomandatory").replace("{0}", errors));
        } else {
            //Save the data contained in the current row in the correct hashmap
            if (action.equals(Constants.ACTION_ADDITION)) {
                this.storeAddition(csvHeader, data, itemClass);
            } else if (action.equals(Constants.ACTION_CLARIFICATION)) {
                this.storeClarification(csvHeader, data, itemClass);
            } else if (action.equals(Constants.ACTION_SUPERSESSION)) {
                this.storeSupersession(data, itemClass);
            } else if (action.equals(Constants.ACTION_INVALIDATION)) {
                this.storeInvalidation(data, itemClass);
            } else if (action.equals(Constants.ACTION_RETIREMENT)) {
                this.storeRetirement(data, itemClass);
            }
        }
    }

    /**
     * This method initialize the list of required fields for each actions
     *
     * @param action
     * @throws Exception
     */
    private int[] requireFieldInitializer(String action) throws Exception {
        int[] requiredFielPosition;
        String tmp;
        String tmps[];

        //Getting properties for mandatory fields for each actions
        tmp = this.properties.getProperty(this.mandatoryFieldsKey + action, null);
        if (tmp == null) {
            throw new Exception(this.localizationres.getString("error.formatmgr.row.nopropertyload").replace("{0}", this.mandatoryFieldsKey + action));
        }
        tmps = tmp.split(this.mandatoryfieldsDelimiter);
        requiredFielPosition = new int[tmps.length];
        for (int i = 0; i < tmps.length; i++) {
            requiredFielPosition[i] = Integer.parseInt(tmps[i]);
        }
        return requiredFielPosition;
    }

    /*
     * The followings methods store the data object in the related hashmap
     */
    private void storeAddition(CsvHeader csvHeader, String[] data, String itemClass) {
        //Creating the addition custom attributes
        ArrayList<CustomattributeDescriptor> customattributes = new ArrayList<CustomattributeDescriptor>();
        ArrayList<Field> tmps = csvHeader.getFields();
        for (int i = 0; i < tmps.size(); i++) {
            Field tmp = tmps.get(i);
            if (tmp.isCustomattribute()) {
                CustomattributeDescriptor customAttribute = new CustomattributeDescriptor(tmp.getName(), data[i], tmp.isRequired(), tmp.isMultivalued(), tmp.isCoded(), tmp.isForeignkey());
                customattributes.add(customAttribute);
            }
        }

        //Creating the addition object
        Addition addition = new Addition(
                data[0],
                data[1],
                data[2],
                data[3],
                data[4],
                data[5],
                data[6],
                data[7],
                data[8],
                customattributes,
                itemClass);

        //Adding the addition to the additions HashMap
        //String key = addition.getLocalId() + Constants.HASHMAP_KEY_SEPARATOR + addition.getCollectionLocalId() + Constants.HASHMAP_KEY_SEPARATOR + addition.getItemClass() + Constants.HASHMAP_KEY_SEPARATOR + addition.getLanguage();
        String key = Addition.getAdditionHashKey(addition);
        log.debug("Creating addition memory hash: " + key);
        if (additions.containsKey(key)) {
            log.info("# Addition, key already in the hashmap: " + key + ", itemclass: " + addition.getItemClass());
        } else {
            additions.put(key, addition);
        }
    }

    private void storeClarification(CsvHeader csvHeader, String[] data, String itemClass) {
        //Creating the clarification custom attributes
        ArrayList<CustomattributeDescriptor> customattributes = new ArrayList<CustomattributeDescriptor>();
        ArrayList<Field> tmps = csvHeader.getFields();
        for (int i = 0; i < tmps.size(); i++) {
            Field tmp = tmps.get(i);
            if (tmp.isCustomattribute()) {
                CustomattributeDescriptor customAttribute = new CustomattributeDescriptor(tmp.getName(), data[i], tmp.isRequired(), tmp.isMultivalued(), tmp.isCoded(), tmp.isForeignkey());
                customattributes.add(customAttribute);
            }
        }

        //Creating the clarification object
        Clarification clarification = new Clarification(
                data[0],
                data[1],
                data[2],
                data[3],
                data[4],
                data[5],
                data[6],
                data[7],
                customattributes,
                itemClass);

        //Adding the clarification to the clarifications HashMap
        String key = Clarification.getClarificationHashKey(clarification.getLocalId(), clarification.getCollectionLocalId(), clarification.getItemClass(), clarification.getLanguage());

        if (clarifications.containsKey(key)) {
            log.info("# Clarification, key already in the hashmap: " + key + ", itemclass: " + clarification.getItemClass());
        } else {
            clarifications.put(key, clarification);
        }
    }

    private void storeSupersession(String[] data, String itemClass) {
        //Creating the supersession object
        Supersession supersession = new Supersession(
                data[0],
                data[1],
                data[2],
                data[3],
                data[4],
                itemClass);

        //Adding the supersession to the supersessions HashMap
        String key = Supersession.getSupersessionHashKey(supersession.getSupersededLocalId(), supersession.getSupersededCollectionLocalId());

        if (supersessions.containsKey(key)) {
            log.info("# Supersession, key already in the hashmap: " + key + ", itemclass: " + supersession.getItemClass());
        } else {
            supersessions.put(key, supersession);
        }
    }

    private void storeInvalidation(String[] data, String itemClass) {
        //Creating the invalidation object
        Invalidation invalidation = new Invalidation(
                data[0],
                data[1],
                data[2],
                data[3],
                data[4],
                itemClass);

        //Adding the invalidation to the invalidations HashMap
        String key = Invalidation.getInvalidationHashKey(invalidation.getLocalId(), invalidation.getCollectionLocalId());

        if (invalidations.containsKey(key)) {
            log.info("# Invalidation, key already in the hashmap: " + key + ", itemclass: " + invalidation.getItemClass());
        } else {
            invalidations.put(key, invalidation);
        }
    }

    private void storeRetirement(String[] data, String itemClass) {
        //Creating the retirement object
        Retirement retirement = new Retirement(
                data[0],
                data[1],
                data[2],
                data[3],
                itemClass);

        //Adding the retirement to the retirements HashMap
        String key = Retirement.getRetirementHashKey(retirement.getLocalId(), retirement.getCollectionLocalId());

        if (retirements.containsKey(key)) {
            log.info("# Retirement, key already in the hashmap: " + key + ", itemclass: " + retirement.getItemClass());
        } else {
            retirements.put(key, retirement);
        }
    }
}
