/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers.actions;

import eu.europa.ec.re3gistrycommon.managers.ChangelogMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributevalueMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemparentMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemsuccessorMgr;
import eu.europa.ec.re3gistrycommon.managers.StatusMgr;
import eu.europa.ec.re3gistrycommon.model.Changelog;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Itemparent;
import eu.europa.ec.re3gistrycommon.model.Itemsuccessor;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Status;
import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.CustomattributeDescriptor;
import eu.europa.ec.re3gistrydata.beans.Invalidation;
import eu.europa.ec.re3gistrydata.constants.Constants;
import eu.europa.ec.re3gistrydata.exceptions.ErrorReport;
import eu.europa.ec.re3gistrydata.exceptions.WarningReport;
import eu.europa.ec.re3gistrydata.managers.DataMgr;
import eu.europa.ec.re3gistrydata.managers.PropertyMgr;
import eu.europa.ec.re3gistrydata.managers.ReportMgr;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class InvalidationMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    private final Properties properties;
    private final EntityManager em;
    private final ReportMgr reportMgr;
    private final DataMgr dataMgr;
    private final Dataprocedure dataprocedure;

    private final String latestVersion;

    //Hashmaps
    private final LinkedHashMap<String, Invalidation> invalidations;

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param em the EntityManager instance
     * @param reportMgr the ReportMgr
     * @param dataMgr
     * @param dataprocedure
     */
    public InvalidationMgr(EntityManager em, ReportMgr reportMgr, DataMgr dataMgr, Dataprocedure dataprocedure) {
        this.em = em;
        this.reportMgr = reportMgr;
        this.dataMgr = dataMgr;
        this.dataprocedure = dataprocedure;

        //Loading module properties
        this.properties = PropertyMgr.getProperties();

        //Initializing hashmap
        this.invalidations = dataMgr.getInvalidation();

        this.latestVersion = properties.getProperty(Constants.KEY_LATEST_VERSION);

        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

    }

    /**
     * Process the invalidations
     *
     * @throws Exception
     */
    public void processInvalidations() throws Exception {
        log.info("# Starting to process invalidations.");
        for (String key : this.invalidations.keySet()) {
            try {
                this.processInvalidation(this.invalidations.get(key));
            } catch (Exception e) {
                reportMgr.addReport(Constants.REPORT_CONTEXT_INVALIDATIONMGR, Constants.REPORT_ACTION_PROCESSINVALIDATION, "This item is skipped: " + key + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ This item is skipped: " + key + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Invalidations processed.");
    }

    private void processInvalidation(Invalidation invalidation) throws Exception {
        //Initializing managers
        StatusMgr statusMgr = new StatusMgr(em);
        ItemMgr itemMgr = new ItemMgr(em);

        //Creating the item UUID;
        String itemUuid = Item.createUuid(invalidation.getLocalId(), this.latestVersion, invalidation.getCollectionLocalId(), invalidation.getItemClass());

        Item item;
        try {
            Status status = statusMgr.getStatusByUuid(Constants.ITEMSTATUS_INVALID);

            try {
                //Checking if the item is in the DB
                item = itemMgr.getItemByUuid(itemUuid);

                //Check if the item is already invalid
                //if (item != null && !item.getStatus().getUuid().equals(Constants.ITEMSTATUS_INVALID)) {
                //Retire any children
                //if (invalidation.isRecursive()) {
                //    this.invalidateChildren(item, status, invalidation);
                //} else {
                //Changing the item's status
                if (!item.getStatus().getUriname().equals(status.getUriname())) {
                    item.setStatus(status);
                    item.setDatetimeinvalidation(new Date());
                    //Updating the item
                    itemMgr.updateItem(item);
                    this.storeInvalidationChangelog(item, invalidation, null);
                } else {
                    log.debug("# Item already invalidated: " + item.getUriname());
                }
                //}

                //Setting successor
                if (invalidation.getSuccessorLocalId() != null && invalidation.getSuccessorLocalId().length() > 0) {
                    this.setSuccessor(item, invalidation, itemMgr);
                }
                //}else{
                //    throw new Exception("The item is already invalid - " + invalidation.getLocalId());
                //}
            } catch (NoResultException ex) {
                throw new Exception(this.localizationres.getString("error.invalidationmgr.stin.noitm").replace("{0}",invalidation.getLocalId()));
            } catch (Exception e) {
                throw new Exception(e.getMessage());
            }
        } catch (Exception e) {
            throw new Exception(this.localizationres.getString("error.invalidationmgr.stin.errgen").replace("{0}", e.getMessage()));
        }
    }

    private void invalidateChildren(Item item, Status status, Invalidation invalidation) throws Exception {
        ItemMgr itemMgr = new ItemMgr(this.em);
        ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);

        //Check if the current item has some children
        List<Itemparent> itemparents = itemparentMgr.findByParent(item);
        if (itemparents.size() > 0) {
            for (Itemparent tmp : itemparents) {
                this.invalidateChildren(tmp.getItem(), status, invalidation);
            }
        }
        this.invalidateCollections(item, status, invalidation);

        //Retire the current item
        if (!item.getStatus().getUriname().equals(status.getUriname())) {
            item.setStatus(status);
            item.setDatetimeinvalidation(new Date());
            itemMgr.updateItem(item);
            this.storeInvalidationChangelog(item, invalidation, null);
        } else {
            log.debug("# Item already invalidated: " + item.getUriname());
        }
    }

    private void invalidateCollections(Item item, Status status, Invalidation invalidation) throws Exception {
        ItemMgr itemMgr = new ItemMgr(this.em);
        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);

        //Check if the current item has some collections
        List<Itemcollection> itemcollections = itemcollectionMgr.findByCollection(item);

        if (itemcollections.size() > 0) {
            for (Itemcollection tmpci : itemcollections) {
                Item tmp = tmpci.getItem();
                if (!item.getStatus().getUriname().equals(status.getUriname())) {
                    tmp.setStatus(status);
                    tmp.setDatetimeinvalidation(new Date());
                    itemMgr.updateItem(tmp);
                    this.storeInvalidationChangelog(tmp, invalidation, null);
                } else {
                    log.debug("# Item already invalidated: " + tmp.getUriname());
                }
            }
        }
    }

    private void setSuccessor(Item item, Invalidation invalidation, ItemMgr itemMgr) throws Exception {
        ItemsuccessorMgr itemsuccessorMgr = new ItemsuccessorMgr(em);
        String successorItemUuid = Item.createUuid(invalidation.getSuccessorLocalId(), this.latestVersion, invalidation.getSuccessorCollectionLocalId(), invalidation.getItemClass());

        //Check if the successor is already ted
        try {
            Itemsuccessor tmpIs = itemsuccessorMgr.getItemsuccessorByUuid(successorItemUuid);
            //if the successor relation is already in the db, skip the insert
            String tmp = this.localizationres.getString("error.invalidationmgr.stin.sadb").replace("{0}", item.getUriname()).replace("{1}", invalidation.getSuccessorLocalId());
            throw new Exception(tmp);
        } catch (Exception e) {
        }

        Item itemSuccessor = null;
        try {
            //Checking if the successor item is in the DB
            itemSuccessor = itemMgr.getItemByUuid(successorItemUuid);
        } catch (NoResultException ex) {
            throw new Exception(this.localizationres.getString("error.invalidationmgr.stin.snav").replace("{0}",invalidation.getLocalId()));
        } catch (Exception e) {
            String tmp = this.localizationres.getString("error.invalidationmgr.stin.serr").replace("{0}",invalidation.getLocalId()).replace("{1}",e.getMessage());
            throw new Exception(tmp);
        }

        //Setting the item-successor association
        Itemsuccessor itemsuccessor = new Itemsuccessor();
        String itemsuccessorUuid = Itemsuccessor.createUuid(item.getUuid(), itemSuccessor.getUuid());
        itemsuccessor.setUuid(itemsuccessorUuid);
        itemsuccessor.setItem(item);
        itemsuccessor.setSuccessor(itemSuccessor);
        itemsuccessorMgr.insertItemsuccessor(itemsuccessor);
    }

    private void storeInvalidationChangelog(Item item, Invalidation invalidation, Languagecode languagecode) throws Exception {
        ChangelogMgr changelogMgr = new ChangelogMgr(em);

        Changelog changelog = new Changelog();
        String changelogUuid = Changelog.createUuid(item, System.currentTimeMillis());

        changelog.setUuid(changelogUuid);
        changelog.setAction(Constants.ACTION_INVALIDATION);
        changelog.setComment(invalidation.getComment());
        changelog.setDataprocedure(this.dataprocedure);
        changelog.setDatecreation(new Date());
        changelog.setItem(item);
        changelog.setLanguagecode(languagecode);

        changelogMgr.insertChangelog(changelog);
    }

    /**
     * ** Analyzer
     *
     ***
     * @throws java.lang.Exception
     */
    public void analyzeInvalidations() throws Exception {
        log.info("# Starting to analyze invalidations.");

        for (String key : this.invalidations.keySet()) {
            Invalidation invalidation = null;
            try {
                invalidation = this.invalidations.get(key);
                this.analyzeInvalidation(invalidation);
            } catch (ErrorReport e) {
                //If an error occurs during the analyzer operation  
                //save a error message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_INVALIDATIONMGR, Constants.REPORT_ACTION_ANALYZEINVALIDATION, "Item: " + ((invalidation != null) ? invalidation.getItemClass() + "/" + invalidation.getLocalId() + ((invalidation.getCollectionLocalId() != null && invalidation.getCollectionLocalId().length() > 0) ? "/" + invalidation.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                log.error("@@ Item: " + ((invalidation != null) ? invalidation.getItemClass() + "/" + invalidation.getLocalId() + ((invalidation.getCollectionLocalId() != null && invalidation.getCollectionLocalId().length() > 0) ? "/" + invalidation.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage());
            } catch (WarningReport e) {
                //If an warningr occurs during the analyzer operation  
                //save a warning message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_INVALIDATIONMGR, Constants.REPORT_ACTION_ANALYZEINVALIDATION, "Item: " + ((invalidation != null) ? invalidation.getItemClass() + "/" + invalidation.getLocalId() + ((invalidation.getCollectionLocalId() != null && invalidation.getCollectionLocalId().length() > 0) ? "/" + invalidation.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.warn("@@ Item: " + ((invalidation != null) ? invalidation.getItemClass() + "/" + invalidation.getLocalId() + ((invalidation.getCollectionLocalId() != null && invalidation.getCollectionLocalId().length() > 0) ? "/" + invalidation.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Invalidations analyzed.");
    }

    private void analyzeInvalidation(Invalidation invalidation) throws ErrorReport, WarningReport {

        ItemMgr itemMgr = new ItemMgr(this.em);
        //Getting the itemclass object
        ItemclassMgr itemclassMgr = new ItemclassMgr(this.em);
        Itemclass itmc = null;
        try {
            itmc = itemclassMgr.getItemclassByUriname(invalidation.getItemClass());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.eitc"));
        }

        //CHecking item availability
        this.checkAvailability(itemMgr, invalidation);

        //Checking children
        this.checkChild(itemMgr, invalidation);

        //Checking collections
        this.checkCollections(itemMgr, invalidation);

        //Checking successor
        this.checkSuccessor(invalidation, itemMgr);

        //Checking linked items
        this.checkLinkedItems(invalidation, itemMgr);
    }

    private void checkAvailability(ItemMgr itemMgr, Invalidation invalidation) throws ErrorReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String uuidItem = Item.createUuid(invalidation.getLocalId(), this.latestVersion, invalidation.getCollectionLocalId(), invalidation.getItemClass());
            Item item = itemMgr.getItemByUuid(uuidItem);

        } catch (NoResultException e) {
            String hashKey = Addition.getAdditionHashKey(invalidation.getLocalId(), invalidation.getCollectionLocalId(), invalidation.getItemClass(), Constants.CHECK_COMMON_LANGUAGE);
            if (!additions.containsKey(hashKey)) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itnav"));
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.erritme").replace("{0}",e.getMessage()));
        }
    }

    private void checkChild(ItemMgr itemMgr, Invalidation invalidation) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String itemUuid = Item.createUuid(invalidation.getLocalId(), this.latestVersion, invalidation.getCollectionLocalId(), invalidation.getItemClass());
            ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);
            Item item = itemMgr.getItemByUuid(itemUuid);
            List<Itemparent> itemChildren = itemparentMgr.findByParent(item);

            if (itemChildren.size() > 0) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itwch").replace("{0}", Integer.toString(itemChildren.size())));
            }

        } catch (NoResultException e) {

        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.erritme").replace("{0}",e.getMessage()));
        }

        //Checking item in the addition datafile
        for (String key : additions.keySet()) {
            try {
                Addition tmpAdd = additions.get(key);

                String itemParentUuid = Item.createUuid(tmpAdd.getParentLocalId(), this.latestVersion, tmpAdd.getCollectionLocalId(), tmpAdd.getItemClass());
                String invalidationUuid = Item.createUuid(invalidation.getLocalId(), this.latestVersion, invalidation.getCollectionLocalId(), invalidation.getItemClass());
                if (itemParentUuid.equals(invalidationUuid)) {
                    throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itwit"));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.erritm").replace("{0}",e.getMessage()));
            }
        }
    }

    private void checkCollections(ItemMgr itemMgr, Invalidation invalidation) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String itemUuid = Item.createUuid(invalidation.getLocalId(), this.latestVersion, invalidation.getCollectionLocalId(), invalidation.getItemClass());
            ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
            Item item = itemMgr.getItemByUuid(itemUuid);
            List<Itemcollection> itemCollections = itemcollectionMgr.findByCollection(item);

            if (itemCollections.size() > 0) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itwcl").replace("{0}",Integer.toString(itemCollections.size())));
            }

        } catch (NoResultException e) {

        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.erritme").replace("{0}",e.getMessage()));
        }

        //Checking item in the addition datafile
        for (String key : additions.keySet()) {
            try {
                Addition tmpAdd = additions.get(key);

                String itemParentUuid = Item.createUuid(tmpAdd.getParentLocalId(), this.latestVersion, tmpAdd.getCollectionLocalId(), tmpAdd.getItemClass());
                String invalidationUuid = Item.createUuid(invalidation.getLocalId(), this.latestVersion, invalidation.getCollectionLocalId(), invalidation.getItemClass());
                if (itemParentUuid.equals(invalidationUuid)) {
                    throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itwpr"));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.errchitcl").replace("{0}", e.getMessage()));
            }
        }
    }

    private void checkSuccessor(Invalidation invalidation, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        //Checking collection
        if (invalidation.getSuccessorLocalId() != null && invalidation.getSuccessorLocalId().length() > 0) {
            try {
                String itemUuid = Item.createUuid(invalidation.getSuccessorLocalId(), this.latestVersion, invalidation.getSuccessorCollectionLocalId(), invalidation.getItemClass());
                Item item = itemMgr.getItemByUuid(itemUuid);

                //Checking if the successor has a valid status
                if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                    String tmp = this.localizationres.getString("error.invalidationmgr.anin.snv").replace("{0}", item.getUriname()).replace("{1}", item.getStatus().getUriname());
                    throw new WarningReport(tmp);
                }
            } catch (NoResultException e) {
                if (!additions.containsKey(Addition.getAdditionHashKey(invalidation.getSuccessorLocalId(), invalidation.getSuccessorCollectionLocalId(), invalidation.getItemClass(), Constants.CHECK_COMMON_LANGUAGE))) {
                    throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.snavdb").replace("{0}",invalidation.getSuccessorLocalId()));
                }
                String tmpKey = Addition.getAdditionHashKey(invalidation.getSuccessorLocalId(), invalidation.getSuccessorCollectionLocalId(), invalidation.getItemClass(), Constants.CHECK_COMMON_LANGUAGE);
                if (additions.containsKey(tmpKey)) {
                    Addition tmpAddition = additions.get(tmpKey);
                    if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                        String tmp = this.localizationres.getString("error.invalidationmgr.anin.snv").replace("{0}", invalidation.getCollectionLocalId()).replace("{1}", tmpAddition.getStatus());
                        throw new WarningReport(tmp);
                    }
                } else {
                    throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.snavdb").replace("{0}",invalidation.getSuccessorLocalId()));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.clcher").replace("{0}",e.getMessage()));
            }
        }
    }

    private void checkLinkedItems(Invalidation invalidation, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();
        CustomattributevalueMgr customattributevalueMgr = new CustomattributevalueMgr(this.em);

        try {
            //Checking elements in the database
            if (customattributevalueMgr.checkCustomattributeByNameValue(invalidation.getItemClass(), invalidation.getLocalId())) {
                throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itpcait"));
            } else {
                //Checking in the addition data file
                for (String key : additions.keySet()) {
                    Addition addition = additions.get(key);
                    ArrayList<CustomattributeDescriptor> customAttributeDescriptors = addition.getCustomAttributeDescriptors();
                    for (CustomattributeDescriptor cad : customAttributeDescriptors) {
                        if (cad.getName().equals(invalidation.getItemClass()) && cad.getValue().equals(invalidation.getLocalId())) {
                            throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.itpcait"));
                        }
                    }
                }
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.invalidationmgr.anin.lncher").replace("{0}",e.getMessage()));
        }

    }
}