/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers.actions;

import eu.europa.ec.re3gistrycommon.managers.ChangelogMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributevalueMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemparentMgr;
import eu.europa.ec.re3gistrycommon.managers.StatusMgr;
import eu.europa.ec.re3gistrycommon.model.Changelog;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Itemparent;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Status;
import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.CustomattributeDescriptor;
import eu.europa.ec.re3gistrydata.beans.Retirement;
import eu.europa.ec.re3gistrydata.constants.Constants;
import eu.europa.ec.re3gistrydata.exceptions.ErrorReport;
import eu.europa.ec.re3gistrydata.exceptions.WarningReport;
import eu.europa.ec.re3gistrydata.managers.DataMgr;
import eu.europa.ec.re3gistrydata.managers.PropertyMgr;
import eu.europa.ec.re3gistrydata.managers.ReportMgr;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class RetirementMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    private final Properties properties;
    private final EntityManager em;
    private final ReportMgr reportMgr;
    private final DataMgr dataMgr;
    private final Dataprocedure dataprocedure;

    private final String latestVersion;

    //Hashmaps
    private final LinkedHashMap<String, Retirement> retirements;

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param em the EntityManager instance
     * @param reportMgr the ReportMgr
     * @param dataMgr
     * @param dataprocedure
     */
    public RetirementMgr(EntityManager em, ReportMgr reportMgr, DataMgr dataMgr, Dataprocedure dataprocedure) {
        this.em = em;
        this.reportMgr = reportMgr;
        this.dataMgr = dataMgr;
        this.dataprocedure = dataprocedure;

        //Loading module properties
        this.properties = PropertyMgr.getProperties();

        //Initializing hashmap
        this.retirements = dataMgr.getRetirement();

        this.latestVersion = properties.getProperty(Constants.KEY_LATEST_VERSION);

        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

    }

    /**
     * Process the retirements
     *
     * @throws Exception
     */
    public void processRetirements() throws Exception {
        log.info("# Starting to process retirements.");
        for (String key : this.retirements.keySet()) {
            try {
                this.processRetirement(this.retirements.get(key));
            } catch (Exception e) {
                //Skipping the current item in case of errors
                reportMgr.addReport(Constants.REPORT_CONTEXT_RETIREMENTMGR, Constants.REPORT_ACTION_PROCESSRETIREMENT, "This item is skipped: " + key + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ This item is skipped: " + key + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Retirements processed.");
    }

    private void processRetirement(Retirement retirement) throws Exception {
        //Initializing managers
        StatusMgr statusMgr = new StatusMgr(em);
        ItemMgr itemMgr = new ItemMgr(em);

        Item item;
        //Creating the item UUID;
        String itemUuid = Item.createUuid(retirement.getLocalId(), this.latestVersion, retirement.getCollectionLocalId(), retirement.getItemClass());
        try {
            Status status = statusMgr.getStatusByUuid(Constants.ITEMSTATUS_RETIRED);

            //Checing if the item is in the DB
            item = itemMgr.getItemByUuid(itemUuid);

            //Retire any children
            if (retirement.isRecursive()) {
                this.retireChildren(item, status, retirement);
            }

            //Changing the item's status
            if (!item.getStatus().getUriname().equals(status.getUriname())) {
                item.setStatus(status);
                item.setDatetimeretirment(new Date());
                itemMgr.updateItem(item);
                this.storeRetirementChangelog(item, retirement, null);
            } else {
                log.debug("# Item already retired: " + item.getUriname());
            }
        } catch (NoResultException ex) {
            throw new Exception(this.localizationres.getString("error.retirementmgr.anrt.itmnf").replace("{0}",retirement.getLocalId()));
        } catch (Exception e) {
            String tmp = this.localizationres.getString("error.retirementmgr.anrt.gerr").replace("{0}", retirement.getLocalId()).replace("{1}",e.getMessage());
            throw new Exception(tmp);
        }
    }

    private void retireChildren(Item item, Status status, Retirement retirement) throws Exception {
        ItemMgr itemMgr = new ItemMgr(this.em);
        ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);

        //Check if the current item has some children
        List<Itemparent> parents = itemparentMgr.findByParent(item);
        if (parents.size() > 0) {
            for (Itemparent tmp : parents) {
                this.retireChildren(tmp.getItem(), status, retirement);
            }
        }
        this.retireCollections(item, status, retirement);

        //Retire the current item
        if (!item.getStatus().getUriname().equals(status.getUriname())) {
            item.setStatus(status);
            item.setDatetimeretirment(new Date());
            itemMgr.updateItem(item);
            this.storeRetirementChangelog(item, retirement, null);
        } else {
            log.debug("# Item already retired: " + item.getUriname());
        }
    }

    private void retireCollections(Item item, Status status, Retirement retirement) throws Exception {
        ItemMgr itemMgr = new ItemMgr(this.em);
        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);

        //Check if the current item has some collections
        List<Itemcollection> itemcollections = itemcollectionMgr.findByCollection(item);

        if (itemcollections.size() > 0) {
            for (Itemcollection tmpic : itemcollections) {
                Item tmp = tmpic.getItem();
                if (!tmp.getStatus().getUriname().equals(status.getUriname())) {
                    tmp.setStatus(status);
                    tmp.setDatetimeretirment(new Date());
                    itemMgr.updateItem(tmp);
                    this.storeRetirementChangelog(tmp, retirement, null);
                } else {
                    log.debug("# Item already retired: " + tmp.getUriname());
                }
            }
        }
    }

    private void storeRetirementChangelog(Item item, Retirement retirement, Languagecode languagecode) throws Exception {
        ChangelogMgr changelogMgr = new ChangelogMgr(this.em);

        Changelog changelog = new Changelog();
        String changelogUuid = Changelog.createUuid(item, System.currentTimeMillis());

        changelog.setUuid(changelogUuid);
        changelog.setAction(Constants.ACTION_RETIREMENT);
        changelog.setComment(retirement.getComment());
        changelog.setDataprocedure(this.dataprocedure);
        changelog.setDatecreation(new Date());
        changelog.setItem(item);
        changelog.setLanguagecode(languagecode);

        changelogMgr.insertChangelog(changelog);

    }

    /**
     * **Analyzer***
     */
    public void analyzeRetirements() throws Exception {
        log.info("# Starting to analyze retirements.");

        for (String key : this.retirements.keySet()) {
            Retirement retirement = null;
            try {
                retirement = retirements.get(key);
                this.analyzeRetirement(retirement);
            } catch (ErrorReport e) {
                //If an error occurs during the analyzer operation  
                //save a error message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_RETIREMENTMGR, Constants.REPORT_ACTION_ANALYZERETIREMENT, "Item: " + ((retirement != null) ? retirement.getItemClass() + "/" + retirement.getLocalId() + ((retirement.getCollectionLocalId() != null && retirement.getCollectionLocalId().length() > 0) ? "/" + retirement.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                log.error("@@ Item: " + ((retirement != null) ? retirement.getItemClass() + "/" + retirement.getLocalId() + ((retirement.getCollectionLocalId() != null && retirement.getCollectionLocalId().length() > 0) ? "/" + retirement.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage());
            } catch (WarningReport e) {
                //If an warningr occurs during the analyzer operation  
                //save a warning message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_RETIREMENTMGR, Constants.REPORT_ACTION_ANALYZERETIREMENT, "Item: " + ((retirement != null) ? retirement.getItemClass() + "/" + retirement.getLocalId() + ((retirement.getCollectionLocalId() != null && retirement.getCollectionLocalId().length() > 0) ? "/" + retirement.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.warn("@@ Item: " + ((retirement != null) ? retirement.getItemClass() + "/" + retirement.getLocalId() + ((retirement.getCollectionLocalId() != null && retirement.getCollectionLocalId().length() > 0) ? "/" + retirement.getCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Retirements analyzed.");
    }

    private void analyzeRetirement(Retirement retirement) throws ErrorReport, WarningReport {

        ItemMgr itemMgr = new ItemMgr(this.em);

        //Getting the itemclass object
        ItemclassMgr itemclassMgr = new ItemclassMgr(this.em);
        Itemclass itmc = null;
        try {
            itmc = itemclassMgr.getItemclassByUriname(retirement.getItemClass());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.egitc"));
        }

        //CHecking item availability
        this.checkAvailability(itemMgr, retirement);

        //Checking children
        this.checkChild(itemMgr, retirement);

        //Checking collections
        this.checkCollections(itemMgr, retirement);

        //Checking linked items
        this.checkLinkedItems(retirement, itemMgr);
    }

    private void checkAvailability(ItemMgr itemMgr, Retirement retirement) throws ErrorReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String uuidItem = Item.createUuid(retirement.getLocalId(), this.latestVersion, retirement.getCollectionLocalId(), retirement.getItemClass());
            Item item = itemMgr.getItemByUuid(uuidItem);

        } catch (NoResultException e) {
            String hashKey = Addition.getAdditionHashKey(retirement.getLocalId(), retirement.getCollectionLocalId(), retirement.getItemClass(), Constants.CHECK_COMMON_LANGUAGE);
            if (!additions.containsKey(hashKey)) {
                throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itnav"));
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itexch").replace("{0}",e.getMessage()));
        }
    }

    private void checkChild(ItemMgr itemMgr, Retirement retirement) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String itemUuid = Item.createUuid(retirement.getLocalId(), this.latestVersion, retirement.getCollectionLocalId(), retirement.getItemClass());
            ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);
            Item item = itemMgr.getItemByUuid(itemUuid);
            List<Itemparent> itemChildren = itemparentMgr.findByParent(item);

            if (itemChildren.size() > 0 && !retirement.isRecursive()) {
                throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itwch").replace("{0}",Integer.toString(itemChildren.size())));
            }

        } catch (NoResultException e) {

        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itexch").replace("{0}",e.getMessage()));
        }

        //Checking item in the addition datafile
        for (String key : additions.keySet()) {
            try {
                Addition tmpAdd = additions.get(key);

                String itemParentUuid = Item.createUuid(tmpAdd.getParentLocalId(), this.latestVersion, tmpAdd.getCollectionLocalId(), tmpAdd.getItemClass());
                String retirementUuid = Item.createUuid(retirement.getLocalId(), this.latestVersion, retirement.getCollectionLocalId(), retirement.getItemClass());
                if (itemParentUuid.equals(retirementUuid)) {
                    throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itpparent"));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.ewcch").replace("{0}",e.getMessage()));
            }
        }
    }

    private void checkCollections(ItemMgr itemMgr, Retirement retirement) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String itemUuid = Item.createUuid(retirement.getLocalId(), this.latestVersion, retirement.getCollectionLocalId(), retirement.getItemClass());
            ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
            Item item = itemMgr.getItemByUuid(itemUuid);
            List<Itemcollection> itemCollections = itemcollectionMgr.findByCollection(item);

            if (itemCollections.size() > 0 && !retirement.isRecursive()) {
                throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itwcl").replace("{0}",Integer.toString(itemCollections.size())));
            }

        } catch (NoResultException e) {

        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itexch").replace("{0}",e.getMessage()));
        }

        //Checking item in the addition datafile
        for (String key : additions.keySet()) {
            try {
                Addition tmpAdd = additions.get(key);

                String itemParentUuid = Item.createUuid(tmpAdd.getParentLocalId(), this.latestVersion, tmpAdd.getCollectionLocalId(), tmpAdd.getItemClass());
                String retirementUuid = Item.createUuid(retirement.getLocalId(), this.latestVersion, retirement.getCollectionLocalId(), retirement.getItemClass());
                if (itemParentUuid.equals(retirementUuid)) {
                    throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.itpparent"));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.erritcl").replace("{0}", e.getMessage()));
            }
        }
    }

    private void checkLinkedItems(Retirement retirement, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();
        CustomattributevalueMgr customattributevalueMgr = new CustomattributevalueMgr(this.em);

        try {
            //Checking elements in the database
            if (customattributevalueMgr.checkCustomattributeByNameValue(retirement.getItemClass(), retirement.getLocalId())) {
                throw new WarningReport(this.localizationres.getString("error.retirementmgr.anrt.impcaf"));
            } else {
                //Checking in the addition data file
                for (String key : additions.keySet()) {
                    Addition addition = additions.get(key);
                    ArrayList<CustomattributeDescriptor> customAttributeDescriptors = addition.getCustomAttributeDescriptors();
                    for (CustomattributeDescriptor cad : customAttributeDescriptors) {
                        if (cad.getName().equals(retirement.getItemClass()) && cad.getValue().equals(retirement.getLocalId())) {
                            throw new WarningReport(this.localizationres.getString("error.retirementmgr.anrt.impcaf"));
                        }
                    }
                }
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.retirementmgr.anrt.erchli").replace("{0}",e.getMessage()));
        }
    }
}
