/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers.actions;

import eu.europa.ec.re3gistrycommon.managers.ChangelogMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributevalueMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemparentMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemsuccessorMgr;
import eu.europa.ec.re3gistrycommon.managers.StatusMgr;
import eu.europa.ec.re3gistrycommon.model.Changelog;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Itemparent;
import eu.europa.ec.re3gistrycommon.model.Itemsuccessor;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Status;
import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.CustomattributeDescriptor;
import eu.europa.ec.re3gistrydata.beans.Supersession;
import eu.europa.ec.re3gistrydata.constants.Constants;
import eu.europa.ec.re3gistrydata.exceptions.ErrorReport;
import eu.europa.ec.re3gistrydata.exceptions.WarningReport;
import eu.europa.ec.re3gistrydata.managers.DataMgr;
import eu.europa.ec.re3gistrydata.managers.PropertyMgr;
import eu.europa.ec.re3gistrydata.managers.ReportMgr;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class SupersessionMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    private final Properties properties;
    private final EntityManager em;
    private final ReportMgr reportMgr;
    private final DataMgr dataMgr;
    private final Dataprocedure dataprocedure;

    private final String latestVersion;

    //Hashmaps
    private final LinkedHashMap<String, Supersession> supersessions;

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param em the EntityManager instance
     * @param reportMgr the ReportMgr
     * @param dataMgr
     * @param dataprocedure
     */
    public SupersessionMgr(EntityManager em, ReportMgr reportMgr, DataMgr dataMgr, Dataprocedure dataprocedure) {
        this.em = em;
        this.reportMgr = reportMgr;
        this.dataMgr = dataMgr;
        this.dataprocedure = dataprocedure;

        //Loading module properties
        this.properties = PropertyMgr.getProperties();

        //Initializing hashmap
        this.supersessions = dataMgr.getSupersession();

        this.latestVersion = properties.getProperty(Constants.KEY_LATEST_VERSION);

        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

    }

    public void processSupersessions() throws Exception {
        log.info("# Starting to process supersessions.");
        for (String key : this.supersessions.keySet()) {
            try {
                this.processSupersession(this.supersessions.get(key));
            } catch (Exception e) {
                reportMgr.addReport(Constants.REPORT_CONTEXT_SUPERSESSIONMGR, Constants.REPORT_ACTION_PROCESSSUPERSESSION, "This item is skipped: " + key + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ This item is skipped: " + key + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Supersessions processed.");
    }

    private void processSupersession(Supersession supersession) throws Exception {
        //Initializing managers
        StatusMgr statusMgr = new StatusMgr(em);
        ItemMgr itemMgr = new ItemMgr(em);

        //Creating the item UUID;
        String supersededItemUuid = Item.createUuid(supersession.getSupersededLocalId(), this.latestVersion, supersession.getSupersededCollectionLocalId(), supersession.getItemClass());
        String newItemUuid = Item.createUuid(supersession.getNewLocalId(), this.latestVersion, supersession.getNewCollectionLocalId(), supersession.getItemClass());

        Status status = statusMgr.getStatusByUuid(Constants.ITEMSTATUS_SUPERSEDED);

        Item supersedeitem;
        Item newitem;

        try {
            //Checing if the item to supersede is in the DB
            supersedeitem = itemMgr.getItemByUuid(supersededItemUuid);

        } catch (NoResultException ex) {
            throw new Exception(this.localizationres.getString("error.supersessionmgr.stsu.itmnfs").replace("{0}", supersession.getSupersededLocalId()));
        } catch (Exception e) {
            String tmp = this.localizationres.getString("error.supersessionmgr.stsu.gerr").replace("{0}", supersession.getSupersededLocalId()).replace("{1}", e.getMessage());
            throw new Exception(tmp);
        }

        //if (supersedeitem != null && !supersedeitem.getStatus().getUuid().equals(Constants.ITEMSTATUS_SUPERSEDED)) {
        try {
            //Checing if the new item is in the DB
            newitem = itemMgr.getItemByUuid(newItemUuid);

        } catch (NoResultException ex) {
            throw new Exception(this.localizationres.getString("error.supersessionmgr.stsu.itmnfs").replace("{0}", supersession.getSupersededLocalId()));
        } catch (Exception e) {
            String tmp = this.localizationres.getString("error.supersessionmgr.stsu.gerr").replace("{0}", supersession.getSupersededLocalId()).replace("{1}", e.getMessage());
            throw new Exception(tmp);
        }

        //Setting the successor
        this.setSuccessor(newitem, supersedeitem);

        //Changing the item's status
        supersedeitem.setStatus(status);
        supersedeitem.setDatetimesupersession(new Date());
        itemMgr.updateItem(supersedeitem);

        this.storeSupersessionChangelog(supersedeitem, supersession, null);
        this.storeSupersessionChangelog(newitem, supersession, null);

        //} else {
        //    throw new Exception("The element is already superseded - " + supersession.getSupersededLocalId());
        //}
    }

    private void setSuccessor(Item newitem, Item supersedeitem) throws Exception {
        ItemsuccessorMgr itemsuccessorMgr = new ItemsuccessorMgr(this.em);
        Itemsuccessor itemsuccessor = new Itemsuccessor();
        String itemsuccessorUuid = Itemsuccessor.createUuid(supersedeitem.getUuid(), newitem.getUuid());

        //Check if the successor is already ted
        try {
            Itemsuccessor tmpIs = itemsuccessorMgr.getItemsuccessorByUuid(itemsuccessorUuid);
            //if the successor relation is already in the db, skip the insert
            String tmp = this.localizationres.getString("error.supersessionmgr.stsu.itradb").replace("{0}",supersedeitem.getUriname()).replace("{1}", newitem.getUriname());
            throw new Exception(tmp);
        } catch (Exception e) {
        }

        itemsuccessor.setUuid(itemsuccessorUuid);
        itemsuccessor.setItem(supersedeitem);
        itemsuccessor.setSuccessor(newitem);
        itemsuccessorMgr.insertItemsuccessor(itemsuccessor);
    }

    private void storeSupersessionChangelog(Item item, Supersession supersession, Languagecode languagecode) throws Exception {
        ChangelogMgr changelogMgr = new ChangelogMgr(em);

        Changelog changelog = new Changelog();
        String changelogUuid = Changelog.createUuid(item, System.currentTimeMillis());

        changelog.setUuid(changelogUuid);
        changelog.setAction(Constants.ACTION_SUPERSESSION);
        changelog.setComment(supersession.getComment());
        changelog.setDataprocedure(this.dataprocedure);
        changelog.setDatecreation(new Date());
        changelog.setItem(item);
        changelog.setLanguagecode(languagecode);

        changelogMgr.insertChangelog(changelog);
    }

    /**
     * ** Analyzer
     *
     ***
     * @throws java.lang.Exception
     */
    public void analyzeSupersessions() throws Exception {
        log.info("# Starting to analyze supersessions.");

        for (String key : this.supersessions.keySet()) {
            Supersession supersession = null;
            try {
                supersession = this.supersessions.get(key);
                this.analyzeSupersession(supersession);
            } catch (ErrorReport e) {
                //If an error occurs during the analyzer operation  
                //save a error message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_SUPERSESSIONMGR, Constants.REPORT_ACTION_ANALYZESUPERSESSION, "Item: " + ((supersession != null) ? supersession.getItemClass() + "/" + supersession.getSupersededLocalId() + ((supersession.getSupersededCollectionLocalId() != null && supersession.getSupersededCollectionLocalId().length() > 0) ? "/" + supersession.getSupersededCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                log.error("@@ Item: " + ((supersession != null) ? supersession.getItemClass() + "/" + supersession.getSupersededLocalId() + ((supersession.getSupersededCollectionLocalId() != null && supersession.getSupersededCollectionLocalId().length() > 0) ? "/" + supersession.getSupersededCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage());
            } catch (WarningReport e) {
                //If an warningr occurs during the analyzer operation  
                //save a warning message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_SUPERSESSIONMGR, Constants.REPORT_ACTION_ANALYZESUPERSESSION, "Item: " + ((supersession != null) ? supersession.getItemClass() + "/" + supersession.getSupersededLocalId() + ((supersession.getSupersededCollectionLocalId() != null && supersession.getSupersededCollectionLocalId().length() > 0) ? "/" + supersession.getSupersededCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.warn("@@ Item: " + ((supersession != null) ? supersession.getItemClass() + "/" + supersession.getSupersededLocalId() + ((supersession.getSupersededCollectionLocalId() != null && supersession.getSupersededCollectionLocalId().length() > 0) ? "/" + supersession.getSupersededCollectionLocalId() : "") : key) + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Supersessions analyzed.");
    }

    private void analyzeSupersession(Supersession supersession) throws ErrorReport, WarningReport {

        ItemMgr itemMgr = new ItemMgr(this.em);
        //Getting the itemclass object
        ItemclassMgr itemclassMgr = new ItemclassMgr(this.em);
        Itemclass itmc = null;
        try {
            itmc = itemclassMgr.getItemclassByUriname(supersession.getItemClass());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.stsu.eritmc"));
        }

        //CHecking item availability
        this.checkAvailability(itemMgr, supersession);

        //Checking children
        this.checkChild(itemMgr, supersession);

        //Checking collections
        this.checkCollections(itemMgr, supersession);

        //Checking successor
        this.checkSuccessor(supersession, itemMgr);

        //Checking linked items
        this.checkLinkedItems(supersession, itemMgr);
    }

    private void checkAvailability(ItemMgr itemMgr, Supersession supersession) throws ErrorReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String uuidItem = Item.createUuid(supersession.getSupersededLocalId(), this.latestVersion, supersession.getSupersededCollectionLocalId(), supersession.getItemClass());
            Item item = itemMgr.getItemByUuid(uuidItem);

        } catch (NoResultException e) {
            String hashKey = Addition.getAdditionHashKey(supersession.getSupersededLocalId(), supersession.getSupersededCollectionLocalId(), supersession.getItemClass(), Constants.CHECK_COMMON_LANGUAGE);
            if (!additions.containsKey(hashKey)) {
                throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.itmnav"));
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.itmexer").replace("{0}",e.getMessage()));
        }
    }

    private void checkChild(ItemMgr itemMgr, Supersession supersession) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String itemUuid = Item.createUuid(supersession.getSupersededLocalId(), this.latestVersion, supersession.getSupersededCollectionLocalId(), supersession.getItemClass());
            ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);
            Item item = itemMgr.getItemByUuid(itemUuid);
            List<Itemparent> itemChildren = itemparentMgr.findByParent(item);

            if (itemChildren.size() > 0) {
                throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.siwch").replace("{0}", Integer.toString(itemChildren.size())));
            }

        } catch (NoResultException e) {

        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.itmexer").replace("{0}",e.getMessage()));
        }

        //Checking element in the addition datafile
        for (String key : additions.keySet()) {
            try {
                Addition tmpAdd = additions.get(key);

                String itemParentUuid = Item.createUuid(tmpAdd.getParentLocalId(), this.latestVersion, tmpAdd.getCollectionLocalId(), tmpAdd.getItemClass());
                String supersessionUuid = Item.createUuid(supersession.getSupersededLocalId(), this.latestVersion, supersession.getSupersededCollectionLocalId(), supersession.getItemClass());
                if (itemParentUuid.equals(supersessionUuid)) {
                    throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.siwpr"));
                }
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.schitch").replace("{0}", e.getMessage()));
            }
        }
    }

    private void checkCollections(ItemMgr itemMgr, Supersession supersession) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String itemUuid = Item.createUuid(supersession.getSupersededLocalId(), this.latestVersion, supersession.getSupersededCollectionLocalId(), supersession.getItemClass());
            ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
            Item item = itemMgr.getItemByUuid(itemUuid);
            List<Itemcollection> itemCollections = itemcollectionMgr.findByCollection(item);

            if (itemCollections.size() > 0) {
                throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.siwcl").replace("{0}",Integer.toString(itemCollections.size())));
            }

        } catch (NoResultException e) {

        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.itmexer").replace("{0}",e.getMessage()));
        }

        //Checking element in the addition datafile
        for (String key : additions.keySet()) {
            try {
                Addition tmpAdd = additions.get(key);

                String itemParentUuid = Item.createUuid(tmpAdd.getParentLocalId(), this.latestVersion, tmpAdd.getCollectionLocalId(), tmpAdd.getItemClass());
                String supersessionUuid = Item.createUuid(supersession.getSupersededLocalId(), this.latestVersion, supersession.getSupersededCollectionLocalId(), supersession.getItemClass());
                if (itemParentUuid.equals(supersessionUuid)) {
                    throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.siwpr"));
                }
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.schitcl").replace("{0}",e.getMessage()));
            }
        }
    }

    private void checkSuccessor(Supersession supersession, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        //Checking collection
        if (supersession.getNewLocalId() != null && supersession.getNewLocalId().length() > 0) {
            try {
                String itemUuid = Item.createUuid(supersession.getNewLocalId(), this.latestVersion, supersession.getNewCollectionLocalId(), supersession.getItemClass());
                Item item = itemMgr.getItemByUuid(itemUuid);

                //Checking if the successor has a valid status
                if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                    String tmp = this.localizationres.getString("error.supersessionmgr.ansu.sscnav").replace("{0}",item.getUriname()).replace("{1}",item.getStatus().getUriname());
                    throw new WarningReport(tmp);
                }
            } catch (NoResultException e) {
                if (!additions.containsKey(Addition.getAdditionHashKey(supersession.getNewLocalId(), supersession.getNewCollectionLocalId(), supersession.getItemClass(), Constants.CHECK_COMMON_LANGUAGE))) {
                    throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.sscnavbl").replace("{0}",supersession.getNewLocalId()));
                }
                String tmpKey = Addition.getAdditionHashKey(supersession.getNewLocalId(), supersession.getNewCollectionLocalId(), supersession.getItemClass(), Constants.CHECK_COMMON_LANGUAGE);
                if (additions.containsKey(tmpKey)) {
                    Addition tmpAddition = additions.get(tmpKey);
                    if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                        String tmp = this.localizationres.getString("error.supersessionmgr.ansu.scnvld").replace("{0}",supersession.getSupersededLocalId()).replace("{1}", tmpAddition.getStatus());
                        throw new WarningReport(tmp);
                    }
                } else {
                    throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.sscnavbl").replace("{0}",supersession.getNewLocalId()));
                }
            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.schcltitm").replace("{0}",e.getMessage()));
            }
        }
    }

    private void checkLinkedItems(Supersession supersession, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();
        CustomattributevalueMgr customattributevalueMgr = new CustomattributevalueMgr(this.em);

        try {
            //Checking elements in the database
            if (customattributevalueMgr.checkCustomattributeByNameValue(supersession.getItemClass(), supersession.getSupersededLocalId())) {
                throw new WarningReport(this.localizationres.getString("error.supersessionmgr.ansu.sitmpfk"));
            } else {
                //Checking in the addition data file
                for (String key : additions.keySet()) {
                    Addition addition = additions.get(key);
                    ArrayList<CustomattributeDescriptor> customAttributeDescriptors = addition.getCustomAttributeDescriptors();
                    for (CustomattributeDescriptor cad : customAttributeDescriptors) {
                        if (cad.getName().equals(supersession.getItemClass()) && cad.getValue().equals(supersession.getSupersededLocalId())) {
                            throw new WarningReport(this.localizationres.getString("error.supersessionmgr.ansu.sitmpfk"));
                        }
                    }
                }
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (WarningReport e) {
            throw new WarningReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.supersessionmgr.ansu.serchlk").replace("{0}",e.getMessage()));
        }
    }
}
