/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 29/09/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller;

import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.model.Registry;
import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import eu.europa.ec.re3gistrystaticizer.constants.Constants;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.ItemStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DefinitionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DescriptionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.LabelAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.RegistryAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.list.RegisterItemStandardList;
import eu.europa.ec.re3gistrystaticizer.servlet.StartStaticizer;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class RegisterStaticizer {

    private final String folderSeparator;
    private final String folderStandardStaticizer;
    private final String folderCustomStaticizer;
    private final EntityManager em;
    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final StartStaticizer startStaticizer;
    private final StaticizeLastImportedItems staticizeLastImportedItems;
    private final Languagecode masterLanguage;

    public RegisterStaticizer(StartStaticizer startStaticizer, EntityManager em, StaticizeLastImportedItems staticizeLastImportedItems) throws Exception {
        this.startStaticizer = startStaticizer;
        this.folderSeparator = startStaticizer.getFolderSeparator();
        this.folderStandardStaticizer = startStaticizer.getFolderStandardStaticizer();
        this.folderCustomStaticizer = startStaticizer.getFolderCustomStaticizer();
        this.staticizeLastImportedItems = staticizeLastImportedItems;
        this.em = em;

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        masterLanguage = languagecodeMgr.getMasterLanguagecode();
    }

    public File addRegisterFiles(Register register, Languagecode languagecode) {
        try {
            LocalizationMgr localizationMgr = new LocalizationMgr(em);
            Localization registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, languagecode);
            if (registerLocalization == null) {
                registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, masterLanguage);
            }
            RegisterStandard registerStandard = addRegisterStandard(register, registerLocalization);
            return createXMLRegisterFile(registerStandard, register);
        } catch (Exception ex) {
            Logger.getLogger(RegisterStaticizer.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public RegisterStandard addRegisterStandard(Register register, Localization registerLocalization) throws Exception {
        RegisterStandard registerStandard = new RegisterStandard();
        registerStandard.setUuid(register.getUuid());
        registerStandard.setBaseuri(register.getBaseuri());
        registerStandard.setUriname(register.getUriname());
        registerStandard.setUri(register.getBaseuri() + "/" + register.getUriname());
        registerStandard.setUristandard(register.getBaseuri() + "/" + register.getUriname());

        registerStandard.setDatecreation(register.getDatecreation());
        registerStandard.setDatelastupdate(register.getDatelastupdate());

        Languagecode registerLanguage = registerLocalization.getLanguage();
        String isocode = registerLanguage.getIsocode();

        LabelAttribute registerLabelAttribute = new LabelAttribute();
        registerLabelAttribute.setLanguage(isocode);
        registerLabelAttribute.setValue(registerLocalization.getLabel());
        registerStandard.setLabel(registerLabelAttribute);

        StringUtils stringUtils = new StringUtils();
        
        if (registerLocalization.getDefinition() != null) {
            DefinitionAttribute definitionAttribute = new DefinitionAttribute();
            definitionAttribute.setLanguage(isocode);
            
            String definition = stringUtils.replaceSpecialCharacters(registerLocalization.getDefinition());
            definitionAttribute.setValue(definition);
            registerStandard.setDefinition(definitionAttribute);
        }

        if (registerLocalization.getDescription() != null) {
            DescriptionAttribute descriptionAttribute = new DescriptionAttribute();
            descriptionAttribute.setLanguage(isocode);
            
            String description = stringUtils.replaceSpecialCharacters(registerLocalization.getDescription());
            descriptionAttribute.setValue(description);
            registerStandard.setDescription(descriptionAttribute);
        }

        registerStandard.setLanguage(registerLanguage);

        registerStandard.setOwner(register.getRegisterowner());
        registerStandard.setManager(register.getRegistermanager());
        registerStandard.setControlbody(register.getRegistercontrolbody());

        LocalizationMgr localizationMgr = new LocalizationMgr(em);

        Registry registry = register.getRegistry();
        RegistryAttribute registryAttribute = new RegistryAttribute();

        LabelAttribute registryLabelAttribute = new LabelAttribute();
        Localization registryLocalization = localizationMgr.findByRegistryAndLanguagecode(registry, registerLanguage);
        String registryIsocode = isocode;
        if (registryLocalization == null) {
            registryLocalization = localizationMgr.findByRegistryAndLanguagecode(registry, masterLanguage);
            registryIsocode = masterLanguage.getIsocode();
        }
        registryLabelAttribute.setValue(registryLocalization.getLabel());
        registryLabelAttribute.setLanguage(registryIsocode);

        registryAttribute.setLabel(registryLabelAttribute);
        registryAttribute.setUri(registry.getBaseuri() + "/" + registry.getUriname());

        registerStandard.setRegistry(registryAttribute);

        RegisterItemStandardList registerItemStandardList = new RegisterItemStandardList();
        ArrayList<ItemStandard> itemStandardList = new ArrayList<ItemStandard>();
        ItemMgr itemMgr = new ItemMgr(em);
        List<Item> items = itemMgr.findItemsForRegister(register);
        for (Iterator<Item> it = items.iterator(); it.hasNext();) {
            Item item = it.next();

            Localization itemLocalization;
            
            try {
                itemLocalization = localizationMgr.findByItemAndLanguagecode(item, registerLanguage);
            } catch (Exception ex) {
                itemLocalization = localizationMgr.findByItemAndLanguagecode(item, masterLanguage);
            }

            ItemStandard itemStandard = staticizeLastImportedItems.createMiniItemStandard(item, itemLocalization, registerLanguage);
            if (itemStandard.getVersionnumber() == 0) {
                itemStandardList.add(itemStandard);
            }
        }

        registerItemStandardList.setItemStandard(itemStandardList);

        registerStandard.setRegisterItemLabelList(registerItemStandardList);
        return registerStandard;
    }

    private File createXMLRegisterFile(RegisterStandard registerStandard, Register register) {
        String isocode = registerStandard.getLabel().getLanguage();

        String registerFolderStandardPath = folderStandardStaticizer + folderSeparator + register.getUriname();
        String registerFolderCustomPath = folderCustomStaticizer + folderSeparator + register.getUriname();
        File registerStandardFolder = new File(registerFolderStandardPath);
        File registercustomFolder = new File(registerFolderCustomPath);

        if (!registerStandardFolder.exists()) {
            registerStandardFolder.mkdirs();
        }
        if (!registercustomFolder.exists()) {
            registercustomFolder.mkdirs();
        }

        String registerUriname = register.getUriname();
        String registerStandardFilePath = registerStandardFolder + folderSeparator + registerUriname + "." + isocode + ".xml";
        File registerStandardFile = new File(registerStandardFilePath);



        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(RegisterStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();

            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            marshallerclassLanguage.marshal(registerStandard, registerStandardFile);

            log.info(
                    "Create with success: " + registerStandardFile);

            /**
             * CUSTOM XML, JSON, HTML, ATOM
             */
            for (String ext
                    : startStaticizer.getEXTENSIONS()) {
                try {
                    String registerXSL = startStaticizer.getStaticizerXSLFolder() + "/" + ext + "/" + registerUriname + "_register." + ext + ".xsl";
                    String registerCustomFilePath = registercustomFolder + folderSeparator + registerUriname + "." + isocode + "." + ext;

                    File finalOutFile = new File(registerCustomFilePath);
                    finalOutFile.createNewFile();
                    File transformerFile = new File(registerXSL);

                    TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                    Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                    transformer.transform(new StreamSource(registerStandardFile), new StreamResult(finalOutFile));

                    log.info("Create with success: " + finalOutFile);
                } catch (Exception ex) {
                    log.error("Error in creation the transformation file: " + ex.getMessage());
                }
            }
        } catch (JAXBException ex) {
            log.error("Error in creation file: " + registerStandardFile + " -JAXBException- " + ex.getMessage());
        }

        return registercustomFolder;
    }
}
